'''
Copyright (c) 2010 Enzo Michelangeli and IT Vision Ltd

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
'''

from numpy import *
from scipy.linalg import lu_factor, lu_solve
from LCPSolve import LCPSolve

def qlcp(Q, e, A=None, b=None, Aeq=None, beq=None, lb=None, ub=None, QI=None):
    '''
    Minimizes e'x + 1/2 x'Q x subject to optional inequality, equality and
    box-bound (converted ti inequality) constraints.
    Note: x is NOT assumed to be non-negative by default.
    This quadratic solver works by converting the QP problem 
    into an LCP problem. It does well up to few hundred variables
    and dense problems (it doesn't take advantage of sparsity).
    If there are equality constraints, the problem may be feasible 
    even when Q is singular. If Q is not singular, it is possible to
    precompute its inverse and pass it as parameter QI (this is
    useful in SQP applications with approximation of the Hessian and
    its inverse, such as DFP or BFGS.
    Returns: x, the solution (or None in case of failure due to ray 
    termination in the LCP solver).
    '''
    nvars = Q.shape[0] # also e.shape[0]
    # convert lb and ub (if present) into Ax <=> b conditions, but
    # skip redundant rows: the ones where lb[i] == -Inf or ub[i] == Inf
    if lb != None:
        delmask = (lb != -Inf)
        addA = compress(delmask, eye(nvars), axis=0)
        addb = compress(delmask, lb, axis=0)
        A = vstack([A, -addA]) if A != None else -addA
        b = concatenate([b, -addb]) if b != None else -addb
    if ub != None:
        delmask = (ub != Inf)
        addA = compress(delmask, eye(nvars), axis=0)
        addb = compress(delmask, ub, axis=0)
        A = vstack([A, addA]) if A != None else addA
        b = concatenate([b, addb]) if b != None else addb

    n_ineq = A.shape[0] if A != None else 0
    #print "nett ineq cons:", n_ineq

    # if there are equality constraints, it's equiv to particular MLCP that 
    # can anyway be converted to LCP 
    '''
    The Karush-Kuhn-Tucker first-order conditions (being mu and lambda the
    KKT multipliers) are:

        (1.1) e + Q x + Aeq' mu + A' lambda = 0
        (1.2) Aeq x = beq
        (1.3) s = b - A x
        (1.4) s >= 0
        (1.5) lambda >= 0   
        (1.6) s' lambda = 0

    lambda are the multipliers of inequality constr., mu of equality constr.,
    and s are the slack variables for inequalities.

    This is a MLCP, where s and lambda are complementary. However, 
    we can re-write (1.1) and (1.2) as:

        | Q  Aeq'| * | x |  =  - | e + A' lambda |
        |-Aeq  0 |   |mu |       |      beq      |

    ...and, as long as 

        | Q  Aeq'|
        |-Aeq  0 |

    ...(in the program called B) is non-singular, we can solve:

        | x | = inv(| Q  Aeq'|) * | -e - A' lambda |
        |mu |       |-Aeq  0 |    |      -beq      |

    Then, if we define:

        M =     | A 0 | * inv( | Q  Aeq'| ) * | A'|
                               |-Aeq  0 |     | 0 |    

        q = b + | A 0 | * inv( | Q  Aeq'| ) * |  e  |
                               |-Aeq  0 |     | beq |

    ...(1.3) can be rewritten as an LCP problem in (s, lambda):

        s = M lambda + q

    (proof: replace M and q in the eq. above, and simplify remembering
    that | A'| lmbd == | A' lmbd |  , finally reobtaining  s = b - Ax )
         | 0 |         |    0    |

    Now, as we saw,    

        | Q  Aeq'| * | x |  = - | e + A' lambda | 
        |-Aeq  0 |   |mu |      |      beq      |

    ...so we can find

        | x | = inv( | Q  Aeq'| ) * - | e + A' lambda |
        |mu |        |-Aeq    |       |      beq      |

    ...and x will be in the first nvar elements of the solution.
    (we can also verify that b - A x == s)

    The advantage of having an LCP in lambda and s alone is also greater efficiency
    trough reduction of dimensionality (no mu's and x's are calculated by the Lemke
    solver). Also, x's are not necessarily positive (unlike s's and lambda's), 
    unless there are eplicit A,b conditions about it. However, the matrix inversion
    does cause a loss of accuracy(which could be estimated through B's condition
    number: should this value be returned with the status?).
    
    '''
    if Aeq != None:
        n_eq = Aeq.shape[0]
        B = vstack([
                hstack([ Q, Aeq.T]),
                hstack([-Aeq, zeros((n_eq, n_eq))])
             ])
        A0 = hstack([A, zeros((n_ineq, n_eq))]) if A != None else None
    else:
        B = Q
        A0 = A

    #print "det(B):", linalg.det(B)
    #print "B's log10(condition number):", log10(linalg.cond(B))

    ee = concatenate((e, beq)) if Aeq != None else e
    
    if A == None: # if no ineq constraints, no need of LCP: just solve a linear system
        xmu = linalg.solve(B, ee)
        x = xmu[:nvars]
    else:   # ve have to compute B's inverse, possibly using Q's inverse (if passed as parameter)
        if QI == None:
            # Even when Q is singular, B might not be, as long as the Eq. Constr. define a suitable subspace
            BI = linalg.inv(B)
        else:  # the inverse of Q was precomputed and passed by the caller (which requires Q not to be singular!)
            if Aeq == None:
                BI = QI
            else:
                # Use formula (1) at http://www.csd.uwo.ca/~watt/pub/reprints/2006-mc-bminv-poster.pdf
                # This is applicable only as long as Q and Aeq.T * Q * Aeq are not singular (i.e.,
                # Q not singular and Aeq full row rank).
                QIAeqT = dot(QI,Aeq.T)
                SQI = linalg.inv(dot(Aeq, QIAeqT))   # inverse of Shur's complement of Q in B
                QIAeqTSQI = dot(QIAeqT,SQI)
                BI = vstack([
                    hstack([QI-dot(dot(QIAeqTSQI,Aeq),QI), -QIAeqTSQI]),
                    hstack([dot(SQI,dot(Aeq,QI)),                 SQI]),
                    ])            
        
        A0BI = dot(A0, BI)
        M = dot(A0BI, A0.T)
        q = b + dot(A0BI, ee)

        # LCP: s = M lambda + q, s >= 0, lambda >= 0, s'lambda = 0
        # print "M is",M.shape,", q is ",q.shape
        s, lmbd, retcode = LCPSolve(M,q) 

        if retcode[0] == 1:
            # xmu = dot(BI, -concatenate([e + dot(A.T, lmbd), beq])) if Aeq != None else dot(BI, -(e + dot(A.T, lmbd)))
            kk = -concatenate([e + dot(A.T, lmbd), beq]) if Aeq != None else -(e + dot(A.T, lmbd))

            xmu = dot(BI, kk)

            x = xmu[:nvars]
        else:
            x = None

    return x
    



from openopt import NLP
from numpy import cos, arange, ones, asarray, abs, zeros, sqrt, sign, asscalar
from pylab import legend, show, plot, subplot, xlabel, subplots_adjust
from string import rjust, ljust, expandtabs, center, lower
from scipy import rand
N = 10
M = 5
s = 1.3
f = lambda x: (abs(x-M) ** s).sum()
df = lambda x: s * sign(x-M) * abs(x-M) ** (s-1)

x0 = cos(arange(N)) #+ rand(N)

c = lambda x: [2* x[0] **4-32, x[1]**2+x[2]**2 - 8]
def dc(x):
    r = zeros((len(c(x0)), p.n))
    r[0,0] = 2 * 4 * x[0]**3
    r[1,1] = 2 * x[1]
    r[1,2] = 2 * x[2]
    return r

K = 1e2
h1 = lambda x: K*(x[-1]-1)**4
h2 = lambda x: (x[-2]-1.5)**4
h3 = lambda x: (x[-5]+x[-6]-2*M+1.5)**6
h = lambda x: (h1(x), h2(x), h3(x))
def dh(x):
    r = zeros((3, N))
    r[0, -1] = 4 * K * (x[-1]-1)**3
    r[1, -2] = 4 * (x[-2]-1.5)**3
    r[2, -5] = 6 * (x[-5]+x[-6]-2*M+1.5)**5
    r[2, -6] = 6 * (x[-5]+x[-6]-2*M+1.5)**5
    return r


lb = -6*ones(N)
ub = 6*ones(N)
lb[3] = 5.5
ub[4] = 4.5
gtol=1e-1
ftol = 1e-6
xtol = 1e-6
diffInt = 1e-8
contol = 1e-6
maxTime = 10
maxIter = 13700
colors = ['b', 'k', 'y', 'g', 'r']

########################################
solvers = ['ralg', 'scipy_cobyla', 'lincher', 'scipy_slsqp']
#solvers = ['scipy_cobyla', 'ralg']
#solvers = ['lincher', 'scipy_cobyla']
solvers = [ 'ralg','scipy_cobyla']
#solvers = [ 'algencan','algencan2']
#solvers = [ 'ralg', 'ralg3']
########################################
colors = colors[:len(solvers)]
lines, results = [], {}
for j in range(len(solvers)):
    solver = solvers[j]
    color = colors[j]
    p = NLP(f, x0, name = 'bench2', df = df, c=c, dc = dc, h=h, dh = dh, lb = lb, ub = ub, gtol=gtol, ftol = ftol, maxFunEvals = 1e7, maxIter = maxIter, maxTime = maxTime,  plot = 1, color = color, iprint = 10, legend = [solvers[j]], show=False,  contol = contol)
#    p = NLP(f, x0, name = 'bench2', df = df, c=c, dc = dc, lb = lb, ub = ub, gtol=gtol, ftol = ftol, maxFunEvals = 1e7, maxIter = 1e4, maxTime = maxTime,  plot = 1, color = color, iprint = 0, legend = [solvers[j]], show=False,  contol = contol)
    if solver[:4] == ['ralg']:
        pass
#        p.gtol = 1e-8
#        p.ftol = 1e-7
#        p.xtol = 1e-7
    elif solver == 'lincher':
        #p.iprint = 1
        p.maxTime = 1e15
        p.maxIter = 100

##    p.check.df = 1
##    p.check.dc = 1
##    p.check.dh = 1
    r = p.solve(solver)
    for fn in ('h','c'):
        if not r.evals.has_key(fn): r.evals[fn]=0 # if no c or h are used in problem
    results[solver] = (r.ff, p.getMaxResidual(r.xf), r.elapsed['solver_time'], r.elapsed['solver_cputime'], r.evals['f'], r.evals['c'], r.evals['h'])
    subplot(2,1,1)
    F0 = asscalar(p.f(p.x0))
    lines.append(plot([0, 1e-15], [F0, F0], color= colors[j]))


for i in range(2):
    subplot(2,1,i+1)
    legend(lines, solvers)

subplots_adjust(bottom=0.2, hspace=0.3)

xl = ['Solver                              f_opt     MaxConstr   Time   CPUTime  fEvals  cEvals  hEvals']

for i in range(len(results)):
    s=(ljust(lower(solvers[i]), 40-len(solvers[i]))+'%0.3f'% (results[solvers[i]][0]) + '       %0.1e' % (results[solvers[i]][1]) + ('      %0.2f'% (results[solvers[i]][2])) + '     %0.2f      '% (results[solvers[i]][3]) + str(results[solvers[i]][4]) + '   ' + rjust(str(results[solvers[i]][5]), 5) + ' '*5 +str(results[solvers[i]][6]))
    xl.append(s)

xl = '\n'.join(xl)
subplot(2,1,1)
xlabel('Time elapsed (without graphic output), sec')

from pylab import *
subplot(2,1,2)
xlabel(xl)
show()




from numpy import asfarray,  ones, all, isfinite, copy, nan, concatenate, dot
from openopt.kernel.ooMisc import WholeRepr2LinConst, xBounds2Matrix
from cvxopt_misc import *
import cvxopt.solvers as cvxopt_solvers

def CVXOPT_QP_Solver(p, solverName):
    if solverName == 'native_CVXOPT_QP_Solver': solverName = None
    cvxopt_solvers.options['maxiters'] = p.maxIter
    cvxopt_solvers.options['feastol'] = p.contol
    cvxopt_solvers.options['abstol'] = p.ftol
    cvxopt_solvers.options['reltol'] = 1e-16
    if p.iprint <= 0: 
        cvxopt_solvers.options['show_progress'] = False
        cvxopt_solvers.options['MSK_IPAR_LOG'] = 0
    xBounds2Matrix(p)
    
    f = copy(p.f).reshape(-1,1)
    
    sol = cvxopt_solvers.qp(Matrix(p.H), Matrix(p.f), Matrix(p.A), Matrix(p.b), Matrix(p.Aeq), Matrix(p.beq), solverName)
    
    p.msg = sol['status']
    if p.msg == 'optimal' :  p.istop = 1000
    else: p.istop = -100
    
    
    if sol['x'] is not None:
        p.xf = xf = asfarray(sol['x']).flatten()
        p.ff = asfarray(0.5*dot(xf, p.matMultVec(p.H, xf)) + p.dotmult(p.f, xf).sum()).flatten()
        p.duals = concatenate((asfarray(sol['y']).flatten(), asfarray(sol['z']).flatten()))
    else:
        p.ff = nan
        p.xf = nan*ones(p.n)



from scipy.optimize import fsolve
from numpy import asfarray
from openopt.kernel.baseSolver import baseSolver


class scipy_fsolve(baseSolver):
    __name__ = 'scipy_fsolve'
    __license__ = "BSD"
    #__authors__ =
    #__alg__ = ""
    __info__ = """
    solves system of n non-linear equations with n variables.
    """

    def __init__(self):pass
    def __solver__(self, p):
        #xf = fsolve(p.f, p.x0, fprime=p.df, xtol = p.xtol, maxfev = p.maxFunEvals, warning = (p.iprint>=0))
        # "warning" has been removed in latest scipy version
        xf = fsolve(p.f, p.x0, fprime=p.df, xtol = p.xtol, maxfev = p.maxFunEvals)
        p.istop = 1000
        p.iterfcn(xf)





from lp_solve import lp_solve as lps
#import lp_solve
#lp_solve.PRESOLVE_DUALS = 0#524288 + 1048576#10000

from openopt.kernel.baseSolver import baseSolver
from numpy import asarray, ones, nan, ravel

from openopt.kernel.ooMisc import LinConst2WholeRepr

def List(x):
    if x is None or x.size == 0: return None
    else: return x.tolist()

class lpSolve(baseSolver):
    __name__ = 'lpSolve'
    __license__ = "LGPL"
    __authors__ = "Michel Berkelaar, michel@es.ele.tue.nl"
    __homepage__ = 'http://sourceforge.net/projects/lpsolve, http://www.cs.sunysb.edu/~algorith/implement/lpsolve/implement.shtml, http://www.nabble.com/lp_solve-f14350i70.html'
    __alg__ = "lpsolve"
    __info__ = 'use p.scale = 1 or True to turn scale mode on'
    __optionalDataThatCanBeHandled__ = ['A', 'Aeq', 'b', 'beq', 'lb', 'ub', 'intVars']
    def __init__(self): pass
    def __solver__(self, p):

        LinConst2WholeRepr(p)
        f = - asarray(p.f) # sign '-' because lp_solve by default searches for maximum, not minimum
        scalemode = False
        if p.scale in [1, True]:
            scalemode = 1
        elif not (p.scale in [None, 0, False]):
            p.warn(self.__name__ + ' requires p.scale from [None, 0, False, 1, True], other value obtained, so scale = 1 will be used')
            scalemode = 1

        [obj, x_opt, duals] = lps(List(f.flatten()), List(p.Awhole), List(p.bwhole.flatten()), List(p.dwhole.flatten()), \
        List(p.lb.flatten()), List(p.ub.flatten()), (1+asarray(p.intVars)).tolist(), scalemode)
        if obj != []:
            # ! don't involve p.ff  - it can be different because of goal and additional constant from FuncDesigner
            p.xf = ravel(x_opt)
            p.duals = duals
            p.istop = 1
            p.msg = 'solved'
        else:
            p.ff = nan
            p.xf = nan*ones(p.n)
            p.duals = []
            p.istop = -1
            p.msg = 'failed'




from scipy.sparse.linalg import lsmr as scipy_lsmr
from openopt.kernel.setDefaultIterFuncs import IS_MAX_ITER_REACHED
from openopt.kernel.baseSolver import baseSolver
from numpy import asfarray, atleast_1d

class lsmr(baseSolver):
    __name__ = 'lsmr'
    __license__ = "BSD"
    __authors__ = 'D. C.-L. Fong and M. A. Saunders'
    __alg__ = '"LSMR: An iterative algorithm for sparse least-squares problems", SIAM J. Sci. Comput., vol. 33, pp. 2950-2971, 2011. http://arxiv.org/abs/1006.0758'
    __info__ = 'requires scipy version >= 0.11b installed'
    atol=1e-06
    btol=1e-06
    conlim=100000000.0
    _canHandleScipySparse = True
    __optionalDataThatCanBeHandled__ = ['damp']
    
    def __init__(self):
        pass
    
    def __solver__(self, p):
        x, istop, itn, normr, normar, norma, conda, normx = \
        scipy_lsmr(p.C, p.d, p.damp if p.damp is not None else 0.0, self.atol, self.btol, self.conlim, p.maxIter)
        xf = x[:p.C.shape[1]]
        ff = atleast_1d(asfarray(p.F(xf)))
        p.xf = p.xk = xf
        p.ff = p.fk = ff
        
        if istop == 0:
            p.istop, p.msg = 0, '0 is solution'
        elif istop == 1:
            p.istop, p.msg = 1000, 'x is an approximate solution to A*x = B according to atol and btol'
        elif istop == 2:
            p.istop, p.msg = 1000, 'x approximately solves the least-squares problem according to atol'
        elif istop == 3:
            p.istop, p.msg = 0, 'COND(A) seems to be greater than CONLIM'
        elif istop == 4:
            p.istop, p.msg =  1000, 'x is an approximate solution to A*x = B with atol = btol = eps (machine precision)'
        elif istop == 5:
            p.istop, p.msg = 1000, 'x approximately solves the least-squares problem according to atol = eps (machine precision)'
        elif istop == 6:
            p.istop, p.msg = 0, 'COND(A) seems to be greater than CONLIM = 1/eps (machine precision)'
        elif istop == 7:
            p.istop, p.msg = IS_MAX_ITER_REACHED, 'max iter (%d) has been reached' % p.maxIter
        




"""
The test is related to obsolete OpenOpt version and doesn't work for now
It was used to write an article related to numerical optimization.
"""

from numpy import *
from openopt import *
#from pylab import *

TestCollection = xrange(5, 26)
#TestCollection = xrange(5, 11)
#TestCollection = [5]
TestCollection = [21]

P = 0
#P = 5e0
xyCoordsAlwaysExist = False

solvers = ['ralg', 'ipopt','scipy_cobyla']
solvers = ['ralg', 'ipopt']
solvers = ['ralg']
#solvers = ['ipopt']
PLOT = 0
Results = {}

contol = 1e-8
xl, yl = 0, 15
yThreshold = -1.0


LeftPointForceX  = 10.00
LeftPointForceY = -3000.00

#maxY = array([yl, 10.0912237188, 5.81736169843, 2.38892359538,-0.528509686607, -3.61027100044,-7.21539449067,-11.0833587513,-14.5626399121,-17.2632362225,-19.3451872935,-21.3081000933,-23.4618325667,-25.641568098,-27.3915251977,-28.4811563214,-29.0663497915,-29.387570995,-29.4874690862,-29.2098877982,-28.3834712745,-27.140937007,-25.7876146522,-24.3568516911,-22.5652835804,-20.1292934475]) + yThreshold

maxY = array([inf, 10.0399786566, 5.61033453009, 2.06002552258, -0.96112448303, -4.13674736433, -7.84375429882, -11.8094052553, -14.9106560481, -17.1598341467, -18.7959512268, -20.240402618, -21.6919090157, -22.9270315304, -23.5750132311, -23.5920241002, -23.2285382362, -22.6103728598, -21.5974976575, -19.9739484665, -17.7441144032, -15.2828386661, -12.9735192373, -10.7235600474, -8.05824588682, -4.62542867851]) + yThreshold
#maxY[1] -= 100

for n in TestCollection:

    oovarInit = oovar('leftPointForces', v0 = [LeftPointForceX, LeftPointForceY], lb=[0, -inf])

    MaxForces = 100*sin(arange(n)) + 5000*ones(n)\
    + array([ -1.37163831e+03,  -1.60694848e+03,  -1.74685759e+03,\
            -1.77099023e+03,  -1.76295446e+03,  -1.83232862e+03,\
            -2.01327466e+03,  -2.23292838e+03,  -2.37990432e+03,\
            -2.40786079e+03,  -2.37604148e+03,  -2.39220174e+03,\
            -2.51227773e+03,  -2.68825834e+03,  -2.81448159e+03,\
            -2.82791948e+03,  -2.76385327e+03,  -2.71873406e+03,\
            -2.75969731e+03,  -2.86237323e+03,  -2.93613565e+03,\
            -2.91260196e+03,  -2.80726185e+03,  -2.69990883e+03,\
            -2.65982371e+03,  -2.68285823e+03,  -2.69793719e+03,\
            -2.63722331e+03,  -2.49886567e+03,  -2.34512448e+03,\
            -2.24405190e+03,  -2.20790597e+03,  -2.18342816e+03,\
            -2.10368051e+03,  -1.95071662e+03,  -1.77036769e+03,\
            -1.62911926e+03,  -1.55345727e+03,  -1.50621766e+03,\
            -1.42212572e+03,  -1.26885753e+03,  -1.07614103e+03,\
            -9.07638583e+02,  -8.02510393e+02,  -7.39726398e+02,\
            -6.58001243e+02,  -5.12877756e+02,  -3.17504802e+02,\
            -1.30014677e+02,   9.99898976e-03])[:n]

    lengths = 5*ones(n)+cos(arange(n))#array([4, 3, 1, 2])
    masses = 15*ones(n)+4*cos(arange(n))#array([10, 15, 20])

    g = 10
    Fm = masses * g

    ########################################################
    s = [20, 20]
    AdditionalMasses = oovar('AdditionalMasses', v0=s + [(100.0-sum(s))/(n-len(s))]*(n-len(s)), lb=zeros(n))
    ########################################################

    ########################################################
    from blockMisc import *

    #def blockEngineFunc(inp, AdditionalMasses, y_limit, blockID):
    def blockEngineFunc(inp, AdditionalMasses, blockID):
        if blockID == 0:
            lFx, lFy, lx, ly, prevBlockForceThreshold, prev_yLimit = inp[0], inp[1], xl, yl, 0, 0
        else:
            lFx, lFy, lx, ly, prevBlockForceThreshold, prev_yLimit = inp[0], inp[1], inp[2], inp[3], inp[4], inp[5]

        prevBlockBroken = blockID>=0 and isnan(prevBlockForceThreshold)\
        or \
        (prevBlockForceThreshold > 0 and P == 0)\
        or \
        (prev_yLimit > 0 or isnan(prev_yLimit) and P == 0 and not xyCoordsAlwaysExist)

        # calculate output
        Fwhole = sqrt(lFx**2+lFy**2)
        ForceThreshold = (Fwhole - MaxForces[blockID]) / 1e4

        CurrentAdditionalMass = AdditionalMasses[blockID]
        rFy = lFy +Fm[blockID] + CurrentAdditionalMass*g # TODO : store Fm[i] inside blocks
        rFx = lFx# Fx are same along whole chain

        dx, dy = lengths[blockID] * lFx/ Fwhole, lengths[blockID] * lFy/ Fwhole
        rx = lx + dx
        ry = ly + dy

        yLimit = ly - maxY[blockID]

        if P != 0:
            projection, distance = project2ball(x = [lFx, lFy], radius=MaxForces[blockID], center = 0)
            ForceThreshold += P * distance / 1e4

            projection, distance = project2box(ly, -inf, maxY[blockID])
            if distance > 0:
                yLimit = P/1e4 * distance

        if prevBlockBroken or (prev_yLimit>0 and P == 0): #and not xyCoordsAlwaysExist:
            rx, ry, ForceThreshold, rFx, rFy, yLimit = nan, nan, nan, nan, nan, nan


        r = array((rFx, rFy, rx, ry, ForceThreshold, yLimit))

        return r

    #def derivative_blockEngineFunc(inp, AdditionalMasses, y_limit, blockID):
    def derivative_blockEngineFunc(inp, AdditionalMasses, blockID):
        # TODO: return nans if prev block is broken
        if blockID == 0:
            lFx, lFy, lx, ly, prevBlockForceThreshold, prev_yLimit = inp[0], inp[1], xl, yl, 0, 0
        else:
            lFx, lFy, lx, ly, prevBlockForceThreshold, prev_yLimit = inp[0], inp[1], inp[2], inp[3], inp[4], inp[5]

        if blockID == 0:
            nVars = 2 + len(AdditionalMasses)
        else:
            nVars = 6 + len(AdditionalMasses)

        #r = zeros((len(inp)+1, nVars))
        r = zeros((6, nVars))

        # d_ rFx /
        r[0, 0] = 1 # d_lFx

        # d_rFy/
        r[1, 1] = 1 # d_lFy
        r[1, len(inp) + blockID] = g
        # TODO: Check it (below)
        #elif blockID < n-1:
            #r[3, 5 + blockID] = g

        # d_rx /
        if blockID!=0: r[2, 2] = 1 # dlx
        Fwhole = sqrt(lFx**2+lFy**2)
        #r[2, 0] = lengths[blockID] / Fwhole # dlFx
        r[2, 0] = lengths[blockID] * lFy**2 / (Fwhole ** 3) # d_lFx
        r[2, 1] = - lengths[blockID] * lFx * lFy / (Fwhole ** 3) # d_lFy

        # d_ry /
        if blockID!=0: r[3, 3] = 1 # dly
        r[3, 0] = - lengths[blockID] * lFx * lFy / (Fwhole ** 3) # d_lFx, and is same to r[2,1]
        r[3, 1] = lengths[blockID] * lFx**2 / (Fwhole ** 3) # d_lFy

        # dForceThreshold
        r[4, 0] = lFx / Fwhole / 1e4 # / dlFx
        r[4, 1] = lFy / Fwhole / 1e4 # / dlFy

        # dYlimit
        if blockID != 0: r[5, 3] = 1.0 #d_Ylimit / d_yl

        if P != 0:
            projection, distance = project2ball(x = [lFx, lFy], radius=MaxForces[blockID], center = 0)
            if distance != 0:
                penalty_derivative = P/1e4 * project2ball_derivative(x = [lFx, lFy], radius=MaxForces[blockID], center = 0)
                r[4, 0] += penalty_derivative[0]
                r[4, 1] += penalty_derivative[1]

            projection, distance = project2box(ly, -inf, maxY[blockID])
            if distance > 0:
                r[5, 3] = P/1e4 * project2box_derivative(ly, -inf, maxY[blockID])

            #projection, distance = project2box(ly, maxY[blockID], inf)
            #yLimit = P * distance
        prevBlockBroken = blockID>=0 and isnan(prevBlockForceThreshold)\
        or \
        (prevBlockForceThreshold > 0 and P == 0)\
        or \
        (prev_yLimit > 0 or isnan(prev_yLimit) and P == 0 and not xyCoordsAlwaysExist)
        if prevBlockBroken:
            r *= nan
        return r

    ooFuncs, c = [], []
    constrYmax = []

    for i in xrange(n):
        oof = oofun(blockEngineFunc, args = copy(i), name = 'blockEngine'+str(i))

        if i == 0:
            oof.input = (oovarInit, AdditionalMasses)
        else:
            oof.input = (ooFuncs[i-1], AdditionalMasses)
        oof.d = derivative_blockEngineFunc
        ooFuncs.append(oof)
        # TODO: replace "4" by named output "ForceThreshold"

        c.append(oolin([0, 0, 0, 0, 1, 0], input = ooFuncs[copy(i)], name='maxForce'+str(i)))
        c.append(oolin([0, 0, 0, 0, 0, 1], input = ooFuncs[copy(i)], name='Ylimit'+str(i)))
        #c.append(y_limit)


#    y_limit = oofun(lambda *inputs: [inp[1]-maxY[i] for i, inp in enumerate(inputs)], input = ooFuncs, name='maxY')
#    def d_y_limit(*inputs):
#        r = zeros((n, len(inputs[0])*n))
#        for i in xrange(n):
#            r[i, len(inputs[0]) * i + 1] = 1
#        return r
#    y_limit.d = d_y_limit
#
#    c.append(y_limit)

    f = oofun(lambda z: z[0]**1.5, input = ooFuncs[-1], d = lambda z:[0, 0, 1.5*z[0]**0.5, 0, 0, 0], name = 'objFunc')
    #f = oofun(lambda z: z[0], input = ooFuncs[-1], d = lambda z:[0, 0, 1, 0, 0,0], name = 'objFunc')
    #f = oolin(array([0, 0, 1, 0, 0, 0]), input = ooFuncs[-1], name = 'objFunc')

    sumOfMasses = oofun(lambda z: 1-z.sum()/100.0, input=AdditionalMasses, d = lambda z: -ones(n)/100.0, name='sOm')
    c.append(sumOfMasses)

    colors = ['b', 'r', 'g', 'y', 'm', 'c']

    for j, solver in enumerate(solvers):

        p = NLP(f, c=c, goal = 'max', gtol = 1e-6,  plot=0, contol = contol, maxFunEvals = 1e10)
        def callback(p):
            print p.c(p.xk)
            return 0
        #p.callback = callback
        if solver == 'scipy_cobyla':
            p.f_iter = max((int(n/2), 5))

        if solver == 'ipopt':
            p.maxIter = 1500 - 40*n
        else:
            p.maxIter = 15000

        r = p.solve(solver,  plot=0, showFeas=1, maxTime = 150, iprint = -1, ftol=1e-6, xtol=1e-6)

        Results[(n, p.solver.__name__)] = r
        if r.isFeasible: msgF = '+'
        else: msgF = '-'
        print 'n=%d' % n, ('f=%3.2f'% r.ff)+'['+msgF+']','Time=%3.1f' % r.elapsed['solver_time']


        if PLOT:
            hold(1)
            for i, oof in enumerate(ooFuncs):
                if i == 0:
                    plot([xl, ooFuncs[0](p.xk)[0]], [yl, ooFuncs[0](p.xk)[1]], colors[j])
                    plot([xl, ooFuncs[0](p.x0)[0]], [yl, ooFuncs[0](p.x0)[1]], 'k')
                else:
                    plot([ooFuncs[i-1](p.xk)[0], ooFuncs[i](p.xk)[0]], [ooFuncs[i-1](p.xk)[1], ooFuncs[i](p.xk)[1]], colors[j])
                    plot([ooFuncs[i-1](p.x0)[0], ooFuncs[i](p.x0)[0]], [ooFuncs[i-1](p.x0)[1], ooFuncs[i](p.x0)[1]], 'k')
    if PLOT: show()



from NLOPT_AUX import NLOPT_AUX
from NLOPT_BASE import NLOPT_BASE
import nlopt
from numpy import isinf

class stogo(NLOPT_BASE):
    __name__ = 'stogo'
    __alg__ = ""
    __optionalDataThatCanBeHandled__ = ['lb', 'ub']
    __isIterPointAlwaysFeasible__ = lambda self, p: True
    _requiresFiniteBoxBounds = True
    #properTextOutput = True
    
    #TODO: check it!
    #_canHandleScipySparse = True
    
    funcForIterFcnConnection = 'f'
#    _requiresBestPointDetection = True
    useRand = True
    
    def __init__(self): pass
    def __solver__(self, p):
        #p.f_iter = 1
        p.maxNonSuccess = 1e10
        p.maxIter = 1e10
        if isinf(p.maxTime):
            s= """currently due to some Python <-> C++ code connection issues 
            the solver stogo requires finite user-defined maxTime; 
            since you have not provided it, 15 sec will be used"""
            p.pWarn(s)
            p.maxTime = 15
        solver = nlopt.GD_STOGO_RAND if self.useRand else nlopt.GD_STOGO
        NLOPT_AUX(p, solver)
        
        #NLOPT_AUX(p, nlopt.GD_MLSL_LDS)



from baseProblem import MatrixProblem
import numpy as np
#from MOP import MOPsolutions

class BPP(MatrixProblem):
    _optionalData = ['objective']
    probType = 'BPP'
    expectedArgs = ['items', 'bins']
    allowedGoals = ['min', 'max', 'minimum', 'maximum']
    showGoal = True
    _immutable = False
    
    def __init__(self, *args, **kw):
        self.goal = 'min'
        self.bins = {}
        #self.objective = ''
        MatrixProblem.__init__(self, *args, **kw)
        self.__init_kwargs = kw
        self._immutable = True
        
    def manage(self, *args, **kw):
        kw['routine'] = 'manage'
        r = self.solve(*args, **kw)
        return r

    def solve(self, *args, **kw):
        import FuncDesigner as fd, openopt as oo 
        
        if len(args) > 1:
            self.err('''
            incorrect number of arguments for solve(), 
            must be at least 1 (solver), other must be keyword arguments''')
        solver = args[0] if len(args) != 0 else kw.get('solver', self.solver)
        routine = kw.pop('routine', 'solve')
        KW = self.__init_kwargs.copy()
        KW.update(kw)
        bins, items = self.bins, self.items
        #n = len(items)
#        objective = KW.get('objective', self.objective)
#        if isinstance(objective, (list, tuple, set)):
#            nCriteria = len(self.objective)
#            if 3 * nCriteria != np.asarray(self.objective).size:
#                objective = [(objective[3*i], objective[3*i+1], objective[3*i+2]) for i in range(int(round(np.asarray(self.objective).size / 3)))]
#            if len(objective) == 1:
#                KW['fTol'], KW['goal'] = objective[0][1:]
#        else:
#            objective = [(self.objective, KW.get('fTol', getattr(self, 'fTol')), KW.get('goal', getattr(self, 'goal')))]

        nCriteria = 1#len(objective)
        isMOP = nCriteria > 1

        #mainCr = objective[0][0]
        
        solverName = solver if type(solver) == str else solver.__name__
        is_interalg = solverName == 'interalg'
        is_glp = False#solverName == 'sa'
        if is_glp:
            assert nCriteria == 1, 'you cannot solve multiobjective KSP by the solver'
            
        #is_interalg_raw_mode = is_interalg and KW.get('dataHandling', oo.oosolver(solver).dataHandling) in ('auto','raw')
        KW.pop('objective', None)
        P = oo.MOP if nCriteria > 1 else oo.GLP if is_interalg else oo.MILP if not is_glp else oo.GLP

        item_numbers = [item.get('n', 1) for item in items]
        nItemTypes = len(items)
        nItems = sum(item_numbers)
        
        Cons = KW.pop('constraints', [])
        if type(Cons) not in (list, tuple):
            Cons = [Cons]
        
        Funcs = {}
        
        usedValues = getUsedValues(Cons)
        bins_keys = set(bins.keys())
        for Set in (bins_keys, usedValues):
            if 'n' in Set:
                Set.remove('n')
        UsedValues = usedValues.copy()
        
        UsedValues.update(bins_keys)
        
        cr_values = {}
        for val in UsedValues:
            if val == 'nItems':
                cr_values[val] = 1
            else:
                cr_values[val] = [obj[val] for obj in items]
        
        nBins = bins.get('n', -1)
        if nBins == -1:
            if len(UsedValues) == 1 and type(bins) == dict:
                Tmp_items = sum(list(cr_values.values()[0]))
                Tmp_bins = bins[list(UsedValues)[0]]
                approx_n_bins = int(np.ceil((2.0 * Tmp_items) / Tmp_bins))
            else:
                approx_n_bins = nItems
            nBins = approx_n_bins
            
        X = fd.oovars(nBins * nItemTypes, domain=int, lb=0).view(np.ndarray).reshape(nItemTypes, nBins)
        
        requireCount  = False
        for item in items:
            if 'n' in item:
#                x[i].domain = np.arange(obj['n']+1) if is_interalg else int
#                x[i].ub = obj['n']
#                x[i].lb = 0
                requireCount  = True
                break
        
        y = fd.oovars(nBins, domain = bool)('y')
        aux_objective = fd.sum(y)
        
        #cr_values = dict([(obj[0], []) for obj in objective])
        
#        MainCr = mainCr if type(mainCr) in (str, np.str_) else list(usedValues)[0]
#        
#        isMainCrMin = objective[0][2] in ('min', 'minimum')
        
        # handling objective(s)
#        FF = []
        
        
        
        for optCrName in UsedValues:
            F = [fd.sum(X[:, j].view(fd.ooarray) * cr_values[optCrName]) for j in range(nBins)]
            Funcs[optCrName] = F
        
        FUNCS = [dict((optCrName, Funcs[optCrName][j]) for optCrName in UsedValues) for j in range(nBins)]
        
#        for obj in objective:
#            FF.append((Funcs[obj[0]](obj[0]) if type(obj[0]) in (str, np.str_) else obj[0](Funcs), obj[1], obj[2]))

        constraints = []
        
        # 1. Constraints from p.constraints

        for c in Cons:
            tmp = [c(_F) for _F in FUNCS]
            constraints += tmp
        
        # 2. Constraints from bins parameters
        for k, v in bins.items():
            if k != 'n':
                constraints += [fd.sum(X[:, j].view(fd.ooarray) * cr_values[k]) <= v*y[j] for j in range(nBins)]
        
        # 3. Number of items of type i from all bins equals to item_numbers[i]
        constraints += [fd.sum(X[i].view(fd.ooarray)) == item_numbers[i] for i in range(nItemTypes)]
        
        # 4. box bounds for interalg
        # TODO: rework when domain=int for interalg will be ready
        if is_interalg:
            for i in range(nItemTypes):
                for v in X[i]:
                    v.domain = np.arange(nItemTypes+1)
#            constraints += [X[i].view(fd.ooarray) <= item_numbers[i] for i in range(nItemTypes)]
        
        startPoint = dict((X[i, j], 0) for i in range(nItemTypes) for j in range(nBins))
        startPoint[y] = [0]*nBins
        
        p = P(aux_objective, startPoint, constraints = constraints)
#        p = P(FF if isMOP else FF[0][0], startPoint, constraints = constraints)#, fixedVars = fixedVars)
        if not isMOP:
            p.goal=self.goal
        
#        r._UsedValues, r._nBins = UsedValues, nBins
#        r._y, r._X, r._items = y, X, items
#        r._optCrName, r._requireCount = optCrName, requireCount
#        r._encode = encode
#        r._encode(r)
        p.finish = lambda r, p: encode(r, UsedValues, nBins, y, X, items, optCrName, requireCount)
        r = getattr(p, routine)(solver, **KW)
        if r is None:
            return
            
        if P != oo.MOP:
            r.ff = p.ff
            
        if isMOP:
            assert 0, 'unimplemented'
#            #assert not requireCount, 'MOP with nItem > 1 is unimplemented yet'
#            r.solution = 'for MOP see r.solutions instead of r.solution'
#            #tmp_c, tmp_v = r.solutions.coords, r.solutions.values  
#            if len(r.solutions):
#                S = []
#                for s in r.solutions:
#                    tmp = [((i, int(s[x[i]])) if requireCount else i) for i in range(n) if s[x[i]]>=1]
#                    if 'name' in items[0]:
#                        tmp = [(items[i]['name'], k) for i, k in tmp] if requireCount else [items[i]['name'] for i in tmp]
#                    S.append(dict(tmp) if requireCount else tmp)
#                Vals = dict([(ff[0].name, r.solutions.values[:, i]) for i, ff in enumerate(FF)])
#
#                Dicts = [s.copy() for s in S]
#                for v in usedValues:
#                    if v != 'nItems':
#                        for i, d in enumerate(Dicts):
#                            d[v] = Vals[v][i]
#                r.solutions = MOPsolutions(Dicts)
#                r.solutions.coords = S
#                r.solutions.values = Vals
        else:
            pass
#            tmp = [((i, int(r.xf[x[i]])) if requireCount else i) for i in range(n) if r.xf[x[i]]>=1]
#            if 'name' in items[0]:
#                tmp = [(items[i]['name'], k) for i, k in tmp] if requireCount else [items[i]['name'] for i in tmp]
#            r.xf = dict(tmp) if requireCount else tmp
        return r

#################################################
#################################################
#################################################

def encode(r, UsedValues, nBins, y, X, items, optCrName, requireCount):
    if requireCount:
        xf = [dict((item.get('name',i), int(r.xf[X[i, j]])) for i, item in enumerate(items) if r.xf[X[i, j]]>0) for j in range(nBins) if r.xf[y[j]]]
    else:
        xf = [tuple(item.get('name',i) for i, item in enumerate(items) if r.xf[X[i, j]]==1) for j in range(nBins) if r.xf[y[j]]]
    
    r.values = dict((optCrName, tuple(sum(item[optCrName]*r.xf[X[i, j]] for i, item in enumerate(items)) for j in range(nBins) if r.xf[y[j]])) for optCrName in UsedValues)
    r.xf = xf
    r.probType = 'BPP'

class D:
    def __init__(self):
        self.used_vals = set()
    def __getitem__(self, item):
        self.used_vals.add(item)
        return 1.0
    
def getUsedValues(Iterator):
    d = D()
    r = set()
    for elem in Iterator:
        if type(elem) in [str, np.str_]:
            r.add(elem)
        else:
            elem(d)
    r.update(d.used_vals)
    return r
     



from baseProblem import MatrixProblem
from STAB import set_routine

class DSP(MatrixProblem):
    _optionalData = []
    probType = 'DSP'
    expectedArgs = ['graph']
    allowedGoals = ['minimum dominating set']
    showGoal = False

    _immutable = False
    
    def __setattr__(self, attr, val): 
        if self._immutable: self.err('openopt DSP instances are immutable, arguments should pass to constructor or solve()')
        self.__dict__[attr] = val

    def __init__(self, *args, **kw):
        MatrixProblem.__init__(self, *args, **kw)
        self.__init_kwargs = kw
        self._immutable = True
    
    def manage(self, *args, **kw):
        kw['routine'] = 'manage'
        r = self.solve(*args, **kw)
        return r
    
    def solve(self, *args, **kw):
        return set_routine(self, *args, **kw)
#        from openopt import STAB
#        KW = self.__init_kwargs
#        KW.update(kw)
#        P = STAB(graph, **KW)
#        r = P.solve(*args)
#        return r
        



# -*- coding: utf-8 -*-
"""
Created on Thu May 05 20:02:00 2011

@author: Tillsten
"""
#from numpy.linalg import norm
#from numpy import dot, asfarray, atleast_1d,  zeros, ones, int, float64, where, inf
import numpy as np
from openopt.kernel.baseSolver import baseSolver

class pymls(baseSolver):
    __name__ = 'pymls'
    __license__ = "BSD"
    __authors__ = 'Till Stensitzki, based directly on code from qcat from Ola Harkegard (http://research.harkegard.se/)'
    __alg__ = ''
    __info__ = ''
    __optionalDataThatCanBeHandled__ = ['lb', 'ub']

    #__bvls_inf__ = 1e100
    T = np.float64
    __pymls_inf__ = 1e9
    def __init__(self): pass

    def __solver__(self, p):
        T = self.T
        A, b = T(p.C),  T(p.d).copy().reshape(-1, 1)
        lb,  ub = p.lb.copy().reshape(-1, 1), p.ub.copy().reshape(-1, 1)
        lb[lb==-np.inf] = -self.__pymls_inf__
        ub[ub==np.inf] = self.__pymls_inf__
        xf = bounded_lsq(A,b,lb,ub)
        p.xf = p.xk = xf.flatten()



from scipy.linalg import qr
eps=np.finfo(float).eps

def mls(B,v,umin,umax,Wv=None,Wu=None,ud=None,u=None,W=None,imax=100):
    """
mls - Control allocation using minimal least squares.

[u,W,iter] = mls_alloc(B,v,umin,umax,[Wv,Wu,ud,u0,W0,imax])

 Solves the bounded sequential least-squares problem

   min ||Wu(u-ud)||   subj. to   u in M

 where M is the set of control signals solving

   min ||Wv(Bu-v)||   subj. to   umin <= u <= umax

 using a two stage active set method. Wu must be diagonal since the
 problem is reformulated as a minimal least squares problem. The
 implementation does not handle the case of coplanar controls.

  Inputs:
  -------
 B     control effectiveness matrix (k x m)
 v     commanded virtual control (k x 1)
 umin  lower position limits (m x 1)
 umax  upper position limits (m x 1)
 Wv    virtual control weighting matrix (k x k) [I]
 Wu    control weighting matrix (m x m), diagonal [I]
 ud    desired control (m x 1) [0]
 u0    initial point (m x 1)
 W0    initial working set (m x 1) [empty]
 imax  max no. of iterations [100]
 
  Outputs:
  -------
 u     optimal control
 W     optimal active set
 iter  no. of iterations (= no. of changes in the working set + 1)

                           0 if u_i not saturated
 Active set syntax: W_i = -1 if u_i = umin_i
                          +1 if u_i = umax_i

Directly Based on the code from:
     Ola Härkegård, www.control.isy.liu.se/~ola   
    see licsence. 
     
"""

  
 #k = number of virtual controls
 #m = number of variables (actuators)

    k,m=B.shape    
    if u==None:
        u=np.mean(umin+umax,0)[:,None]                
    if W==None:
        W=np.zeros((m,1))
    if ud==None:
        ud=np.zeros((m,1))
    if Wu==None:
        Wu=np.eye(m)
    if Wv==None:
        Wv=np.eye(k)
    
    phase=1
    
    #Reformulate as a minimal least squares problem. See 2002-03-08 (1).    
    A=Wv.dot(B).dot(np.linalg.pinv(Wu))    
    #print B, v
    #A=B    
    b = Wv.dot(v-B.dot(ud))
    #b=v
    #print b
    xmin = (umin-ud)
    xmax = (umax-ud)
    # Compute initial point and residual.
    x = Wu.dot(u-ud)
    #x#=umin-umax
    r = A.dot(x)-b
#    print x.shape, r.shape, b.shape,x,r
    #Determine indeces of free variables
    i_free = W==0    
    m_free = np.sum(i_free)
    
    for i in range(imax):  
        #print 'Iter: ', i 
        if phase==1:
            A_free = A[:,i_free.squeeze()]
            if m_free<=k:                
                if m_free>0:
                    p_free=np.linalg.lstsq(-A_free,r)[0]                
            else:                
                q1,r1=qr(A_free.T)
                p_free=-q1.dot(np.solve(r1.T,r))
            p=np.zeros((m,1))
            p[i_free.squeeze()]=p_free
            
        
        else:            
            i_fixed=np.logical_not(i_free)
            m_fixed=m-m_free
            
            if m_fixed>0:            
                HT=U[i_fixed.squeeze(),:].T                      
                V,Rtot= qr(np.atleast_2d(HT))
                V1=V[:,:m_fixed]
                V2=V[:,m_fixed+1:]
                R=Rtot[:,m_fixed]      
            else:
                V,Rtot=np.array([[]]),np.array([[]])
                V1=V2=R=V.T
            
            s=-V2.T.dot(z)
            pz=V2.dot(s)
            p=U.dot(pz)
            
        x_opt=x+p
        infeasible=np.logical_or(x_opt<xmin,x_opt>xmax)
        if not np.any(infeasible[i_free]):
            x=x_opt
            
            if phase==1:
                r=r+A.dot(p)                
            else:
                z=z+pz
            if phase==1 and m_free>=k:
                phase=2
                Utot, Stot=qr(A.T)
                U=Utot[:,k:]
                z=U.T.dot(x)
                
                
            else:            
                lam=np.zeros((m,1))
                if m_free<m:
                    if phase==1:
                        g=A.T.dot(r)
                        lam=-W*g
                    else:
                        lam[i_fixed]=-W[i_fixed]*np.solve(R,V1.T.dot(z))
                    
                    if np.all(lam>= -eps):
                        u=np.linalg.solve(Wu,x)+ud
                        return u
                
                lambda_neg,i_neg=np.min(lam),np.argmin(lam)
                W[i_neg]=0
                i_free[i_neg]=1
                m_free+=1
        else:
            dist=np.ones((m,1))            
            
            i_min=np.logical_and(i_free,p<0)
            i_max=np.logical_and(i_free,p>0)
            
            
                
            dist[i_min]=(xmin[i_min]-x[i_min])/p[i_min]
            dist[i_max]=(xmax[i_max]-x[i_max])/p[i_max]
                
            
            alpha,i_alpha=np.min(dist),np.argmin(dist)
            
            x = x + alpha*p
            if phase==1:
                r=r+A.dot(alpha*p) #!!                
            else:
                z=z+alpha*pz
            W[i_alpha]=np.sign(p[i_alpha])
            i_free[i_alpha]=0
            m_free-=1
    u=np.linalg.solve(Wu,x)+ud
    return u
    
def bounded_lsq(A,b,lower_lim,upper_lim):
    """
    Minimizes:
    
    |Ax-b|_2 

    for lower_lim<x<upper_lim.
    """
    return mls(A,b,lower_lim,upper_lim)

#if __name__=='__main__':
#    from numpy.core.umath_tests import matrix_multiply
#    import matplotlib.pyplot as plt
#    plt.rcParams['font.family']='serif'
#    A=np.array([[1,-3],[5,7]])
#    b=np.array([[-50],[50]])
#    ll=np.array(([[-10],[-10]]))
#    ul=np.array(([[10],[10]]))
#    
#    Ud=np.array(([0,0]))
#    gamma=1000
#    
#    
#    x0=bounded_lsq(A,b,ll,ul)
#    x=np.linspace(-30,30,500)
#    y=np.linspace(-30,30,500)
#    
#    
#    X,Y=np.meshgrid(x,y)
#    
#    S=np.dstack((X,Y))
#    SN=matrix_multiply(S,A.T)
#    plt.clf()
#   
#    plt.contourf(x,y,np.sqrt(((SN-b.T)**2).sum(-1)),30,cmap=plt.cm.PuBu)
#    plt.colorbar()
#    #plt.axhline(ll[0])
#    #plt.axhline(ul[0])
#    #plt.axvline(ll[1])
#    #plt.axvline(ul[1])
#    rect=np.vstack((ll,ul-ll))
#    patch=plt.Rectangle(ll,*(ul-ll),facecolor=(0.0,0.,0.,0))
#    plt.gca().add_patch(patch)
#    plt.annotate("Bounded Min",
#                xy=x0, xycoords='data',
#                xytext=(-5, 5), textcoords='data',
#                arrowprops=dict(arrowstyle="->",
#                                connectionstyle="arc3"),
#                )
#    
#    plt.annotate("Lsq Min",
#                xy=np.linalg.lstsq(A,b)[0], xycoords='data',
#                xytext=(20, 10), textcoords='offset points',
#                arrowprops=dict(arrowstyle="->",
#                                connectionstyle="arc3"),
#                )
#                
#    plt.scatter(*x0)
#    plt.scatter(*np.linalg.lstsq(A,b)[0])
#    plt.show()
#    
    
            
    
        
        



__docformat__ = "restructuredtext en"
from numpy import asfarray, array, asarray, argmax, zeros, isfinite, all, isnan, arange

empty_arr = asfarray([])

try:
    from scipy.sparse import csr_matrix
except:
    pass
    
#def MULT(A, x):
#    if isinstance(A, ndarray):
#        return dot(A, x)
#    else:
#        t2 = csc_matrix(x)
#        if t2.shape[0] != A.shape[1]:
#            if t2.shape[1] == A.shape[1]:
#                t2 = t2.T
#        return A._mul_sparse_matrix(t2).toarray()         


class residuals:
    def __init__(self):
        pass
    def _get_nonLinInEq_residuals(self, x):
        if hasattr(self.userProvided, 'c') and self.userProvided.c: return self.c(x)
        else: return empty_arr.copy()

    def _get_nonLinEq_residuals(self, x):
        if hasattr(self.userProvided, 'h') and self.userProvided.h: return self.h(x)
        else: return empty_arr.copy()

    def _get_AX_Less_B_residuals(self, x):
        if self.A is not None and self.A.size > 0: 
            if x.ndim > 1: # multiarray
                return (self.matmult(self.A, x.T) - self.b.reshape(-1, 1)).T if not hasattr(self, '_A') else \
                (self._A._mul_sparse_matrix(csr_matrix(x.T)).toarray().T - self.b.reshape(-1, 1)).T
            return self.matmult(self.A, x).flatten() - self.b if not hasattr(self, '_A') else \
            self._A._mul_sparse_matrix(csr_matrix((x, (arange(self.n), zeros(self.n))), shape=(self.n, 1))).toarray().flatten() - self.b
            #return self.matmult(self.A, x).flatten() - self.b if not hasattr(self, '_A') else self._A._mul_sparse_matrix(csr_matrix(x).reshape((self.n, 1))).toarray().flatten() - self.b
        else: return empty_arr.copy()

    def _get_AeqX_eq_Beq_residuals(self, x):
        if self.Aeq is not None and self.Aeq.size>0 : 
            if x.ndim > 1: # multiarray
                return (self.matmult(self.Aeq, x.T) - self.beq.reshape(-1, 1)).T if not hasattr(self, '_Aeq') else \
                (self._Aeq._mul_sparse_matrix(csr_matrix(x.T)).toarray().T - self.beq.reshape(-1, 1)).T
            return self.matmult(self.Aeq, x).flatten() - self.beq if not hasattr(self, '_Aeq') else \
            self._Aeq._mul_sparse_matrix(csr_matrix((x, (arange(self.n), zeros(self.n))), shape=(self.n, 1))).toarray().flatten() - self.beq
        else: return empty_arr.copy()

    def _getLbresiduals(self, x):
        return self.lb - x

    def _getUbresiduals(self, x):
        return x - self.ub

    def _getresiduals(self, x):
#        if 'x' in self.prevVal['r'].keys() and all(x == self.prevVal['r']['x']):
#            return self.prevVal['r']['Val']
        # TODO: add quadratic constraints
        r = EmptyClass()
        # TODO: simplify it!
        if self._baseClassName == 'NonLin':
            r.c = self._get_nonLinInEq_residuals(x)
            r.h = self._get_nonLinEq_residuals(x)
        else:
            r.c = r.h = 0
        r.lin_ineq = self._get_AX_Less_B_residuals(x)
        r.lin_eq= self._get_AeqX_eq_Beq_residuals(x)
        r.lb = self._getLbresiduals(x)
        r.ub = self._getUbresiduals(x)
#        self.prevVal['r']['Val'] = deepcopy(r)
#        self.prevVal['r']['x'] = copy(x)
        return r

    def getMaxResidual(self, x, retAll = False):
        """
        if retAll:  returns
        1) maxresidual
        2) name of residual type (like 'lb', 'c', 'h', 'Aeq')
        3) index of the constraint of given type
        (for example 15, 'lb', 4 means maxresidual is equal to 15, provided by lb[4])
        don't forget about Python indexing from zero!
        if retAll == False:
        returns only r
        """

        residuals = self._getresiduals(x)
        r, fname, ind = 0, None, None
        for field in ('c',  'lin_ineq', 'lb', 'ub'):
            fv = asarray(getattr(residuals, field)).flatten()
            if fv.size>0:
                ind_max = argmax(fv)
                val_max = fv[ind_max]
                if r < val_max:
                    r, ind, fname = val_max, ind_max, field
        for field in ('h', 'lin_eq'):
            fv = asarray(getattr(residuals, field)).flatten()
            if fv.size>0:
                fv = abs(fv)
                ind_max = argmax(fv)
                val_max = fv[ind_max]
                if r < val_max:
                    r, ind, fname = val_max, ind_max, field
#        if self.probType == 'NLSP':
#            fv = abs(self.f(x))
#            ind_max = argmax(fv)
#            val_max = fv[ind_max]
#            if r < val_max:
#                r, ind, fname = val_max, ind_max, 'f'
        if retAll:
            return r, fname, ind
        else:
            return r

    def _getMaxConstrGradient2(self, x):
        g = zeros(self.n)
        mr0 = self.getMaxResidual(x)
        for j in range(self.n):
            x[j] += self.diffInt
            g[j] = self.getMaxResidual(x)-mr0
            x[j] -= self.diffInt
        g /= self.diffInt
        return g

    def getMaxConstrGradient(self, x,  retAll = False):
        g = zeros(self.n)
        maxResidual, resType, ind = self.getMaxResidual(x, retAll=True)
        if resType == 'lb':
            g[ind] -= 1 # N * (-1), -1 = dConstr/dx = d(lb-x)/dx
        elif resType == 'ub':
            g[ind] += 1 # N * (+1), +1 = dConstr/dx = d(x-ub)/dx
        elif resType == 'A':
            g += self.A[ind]
        elif resType == 'Aeq':
            rr = self.matmult(self.Aeq[ind], x)-self.beq[ind]
            if rr < 0:  g -= self.Aeq[ind]
            else:  g += self.Aeq[ind]
        elif resType == 'c':
            dc = self.dc(x, ind).flatten()
            g += dc
        elif resType == 'h':
            dh = self.dh(x, ind).flatten()
            if self.h(x, ind) < 0:  g -= dh#CHECKME!!
            else: g += dh#CHECKME!!
        if retAll:
            return g,  resType,  ind
        else:
            return g

#    def _getLagrangeresiduals(self, x, lm):
#        #lm is Lagrange multipliers
#        residuals = self.getresiduals(x)
#        r = 0
#
#        for field in ['c', 'h', 'A', 'Aeq', 'lb', 'ub']:
#            fv = getattr(residuals, field)
#            if fv not in ([], ()) and fv.size>0: r += self.dotwise(fv, getattr(lm, field))
#        return r
#        #return r.nonLinInEq * lm.nonLinInEq + r.nonLinEq * lm.nonLinEq + \
#                   #r.aX_Less_b * lm.aX_Less_b + r.aeqX_ineq_beq * lm.aeqX_ineq_beq + \
#                   #r.res_lb * lm.res_lb + r.res_ub * lm.res_ub

    def isFeas(self, x):
        if any(isnan(self._get_nonLinEq_residuals(x))) or any(isnan(self._get_nonLinInEq_residuals(x))):
            return False
        is_X_finite = all(isfinite(x))
        is_ConTol_OK = self.getMaxResidual(x) <= self.contol
        cond1 = is_ConTol_OK and is_X_finite and all(isfinite(self.objFunc(x)))
        if self.probType in ('NLSP', 'SNLE'): return cond1 and self.F(x) < self.ftol
        else: return cond1

    def discreteConstraintsAreSatisfied(self, x):
        k = -1
        A = array([0, 1])
        for i in self.discreteVars.keys():#range(m):	# check x-vector
            # TODO: replace it by "for i, val in dict.itervalues()"
            s = self.discreteVars[i] if self.discreteVars[i] is not bool and self.discreteVars[i] is not 'bool' else A
            if not any(abs(x[i] - s) < self.discrtol):
                k=i	# Violation of this set constraint.
                break # Go and split for this x-component
        if k == -1:
            return True
        else:
            return False
        
class EmptyClass:
    pass




from openopt import NLP
from numpy import cos, arange, ones, asarray, abs, zeros
N = 30
M = 5
ff = lambda x: ((x-M)**2).sum()
p = NLP(ff, cos(arange(N)))
p.df =  lambda x: 2*(x-M)
p.c = lambda x: [2* x[0] **4-32, x[1]**2+x[2]**2 - 8]

def dc(x):
    r = zeros((2, p.n))
    r[0,0] = 2 * 4 * x[0]**3
    r[1,1] = 2 * x[1]
    r[1,2] = 2 * x[2]
    return r
p.dc = dc

h1 = lambda x: 1e1*(x[-1]-1)**4
h2 = lambda x: (x[-2]-1.5)**4
p.h = lambda x: (h1(x), h2(x))

def dh(x):
    r = zeros((2, p.n))
    r[0,-1] = 1e1*4*(x[-1]-1)**3
    r[1,-2] = 4*(x[-2]-1.5)**3
    return r
p.dh = dh

p.lb = -6*ones(N)
p.ub = 6*ones(N)
p.lb[3] = 5.5
p.ub[4] = 4.5

#r = p.solve('ipopt', showLS=0, xtol=1e-7, maxIter = 1504)
#solver = 'ipopt'
solver = 'ralg'
#solver = 'scipy_slsqp'
#solver = 'algencan'
r = p.solve(solver, maxIter = 1504, plot=1)
#!! fmin_cobyla can't use user-supplied gradient
#r = p.solve('scipy_cobyla')




from numpy import array, nan, atleast_1d, copy, vstack, hstack, isscalar, ndarray, prod, int32
import numpy as np
from setDefaultIterFuncs import USER_DEMAND_EXIT
from ooMisc import killThread, setNonLinFuncsNumber
from nonOptMisc import scipyInstalled, Vstack, isspmatrix, isPyPy
try:
    from DerApproximator import get_d1
    DerApproximatorIsInstalled = True
except:
    DerApproximatorIsInstalled = False


class nonLinFuncs:
    def __init__(self): pass

    def wrapped_func(p, x, IND, userFunctionType, ignorePrev, getDerivative):#, _linePointDescriptor = None):
        if isinstance(x, dict):
            if not p.isFDmodel: p.err('calling the function with argument of type dict is allowed for FuncDesigner models only')
            x = p._point2vector(x)
        if not getattr(p.userProvided, userFunctionType): return array([])
        if p.istop == USER_DEMAND_EXIT:
            if p.solver.useStopByException:
                raise killThread
            else:
                return nan                
                
        if getDerivative and not p.isFDmodel and not DerApproximatorIsInstalled:
            p.err('For the problem you should have DerApproximator installed, see http://openopt.org/DerApproximator')

        #userFunctionType should be 'f', 'c', 'h'
        funcs = getattr(p.user, userFunctionType)
        #funcs_num = getattr(p, 'n'+userFunctionType)
        if IND is not None:
            ind = p.getCorrectInd(IND)
        else: ind = None

        # this line had been added because some solvers pass tuple instead of
        # x being vector p.n x 1 or matrix X=[x1 x2 x3...xk], size(X)=[p.n, k]
        if not isspmatrix(x): # mb for future purposes
            x = atleast_1d(x)
#            if not str(x.dtype).startswith('float'):
#                x = asfarray(x)
        else:
            if p.debug:
                p.pWarn('[oo debug] sparse matrix x in nonlinfuncs.py has been encountered')
        
#        if not ignorePrev: 
#            prevKey = p.prevVal[userFunctionType]['key']
#        else:
#            prevKey = None
#            
#        # TODO: move it into runprobsolver or baseproblem
#        if p.prevVal[userFunctionType]['val'] is None:
#            p.prevVal[userFunctionType]['val'] = zeros(getattr(p, 'n'+userFunctionType))
#
#        if prevKey is not None and p.iter > 0 and array_equal(x,  prevKey) and ind is None and not ignorePrev:
#            #TODO: add counter of the situations
#            if not getDerivative:
#                r = copy(p.prevVal[userFunctionType]['val'])
#                #if p.debug: assert array_equal(r,  p.wrapped_func(x, IND, userFunctionType, True, getDerivative))
#                if ind is not None: r = r[ind]
#                
#                if userFunctionType == 'f':
#                    if p.isObjFunValueASingleNumber: r = r.sum(0)
#                    if p.invertObjFunc: r = -r
#                    if  p.solver.funcForIterFcnConnection=='f' and any(isnan(x)):
#                        p.nEvals['f'] += 1
#
#                        if p.nEvals['f']%p.f_iter == 0:
#                            p.iterfcn(x, fk = r)
#                return r

        args = getattr(p.args, userFunctionType)

        # TODO: handle it in prob prepare
        if not hasattr(p, 'n'+userFunctionType): setNonLinFuncsNumber(p,  userFunctionType)

#        if ind is None:
#            nFuncsToObtain = getattr(p, 'n'+ userFunctionType)
#        else:
#            nFuncsToObtain = len(ind)

        if x.shape[0] != p.n and (x.ndim<2 or x.shape[1] != p.n): 
            p.err('x with incorrect shape passed to non-linear function')

        #TODO: code cleanup (below)
        if getDerivative or x.ndim <= 1 or x.shape[0] == 1:
            nXvectors = 1
            x_0 = copy(x)
        else:
            nXvectors = x.shape[0]

        # TODO: use certificate instead 
        if p.isFDmodel:
            if getDerivative:
                if p.freeVars is None or (p.fixedVars is not None and len(p.freeVars) < len(p.fixedVars)):
                    funcs2 = [(lambda x, i=i: \
                      p._pointDerivative2array(
                                               funcs[i].D(x, Vars = p.freeVars, useSparse=p.useSparse, fixedVarsScheduleID=p._FDVarsID, exactShape=True), 
                                               useSparse=p.useSparse, func=funcs[i], point=x)) \
                      for i in range(len(funcs))]
                else:
                    funcs2 = [(lambda x, i=i: \
                      p._pointDerivative2array(
                                               funcs[i].D(x, fixedVars = p.fixedVars, useSparse=p.useSparse, fixedVarsScheduleID=p._FDVarsID, exactShape=True), 
                                               useSparse=p.useSparse, func=funcs[i], point=x)) \
                      for i in range(len(funcs))]
            else: # function, not derivative
                _kw = {'fixedVarsScheduleID': p._FDVarsID}
                if p.freeVars is None or (p.fixedVars is not None and len(p.freeVars) < len(p.fixedVars)):
                    _kw['Vars'] = p.freeVars
                else:
                    _kw['fixedVars'] = p.fixedVars
                funcs2 = [(lambda x, i=i: \
                               funcs[i]._getFuncCalcEngine(x, **_kw))\
                      for i in range(len(funcs))]
        else: # not FD model
            funcs2 = funcs
            
        if ind is None: 
            Funcs = funcs2
        elif ind is not None and p.functype[userFunctionType] == 'some funcs R^nvars -> R':
            Funcs = [funcs2[i] for i in ind]
        else:
            Funcs = getFuncsAndExtractIndexes(p, funcs2, ind, userFunctionType)

#        agregate_counter = 0
        
        Args = () if p.isFDmodel else args
            
        if nXvectors == 1:
            if p.isFDmodel:
                X = p._vector2point(x) 
                X._p = p
                #X._linePointDescriptor = _linePointDescriptor
            else:
                X = x
        
        if nXvectors > 1: # and hence getDerivative isn't involved
            #temporary, to be fixed
            if userFunctionType == 'f':
               assert p.isObjFunValueASingleNumber
            

            if p.isFDmodel:
                assert ind is None
                if isPyPy or p.hasVectorizableFuncs: # TODO: get rid of box-bound constraints
                    from FuncDesigner.ooPoint import ooPoint as oopoint
                    from FuncDesigner.multiarray import multiarray
                    
                    # TODO: new
                    xx = []
                    counter = 0
                    #xT = x.T
                    for i, oov in enumerate(p._freeVarsList):
                        s = p._optVarSizes[oov]
                        xx.append((oov, (x[:, counter: counter + s].flatten() if s == 1 else x[:, counter: counter + s]).view(multiarray)))
#                        xx.append((oov, multiarray(x[:, counter: counter + s].flatten() if s == 1 else x[:, counter: counter + s])))
                        counter += s
                    X = oopoint(xx)
                    X.dictOfFixedFuncs = p.dictOfFixedFuncs
                    X._dictOfRedirectedFuncs = p._dictOfRedirectedFuncs
                    X.maxDistributionSize = p.maxDistributionSize
                    X._dictOfStochVars = p._dictOfStochVars
                    X._p = p
                    if x.ndim > 1:
                        X.isMultiPoint = True
                        X.N = nXvectors
                if len(p.unvectorizableFuncs) != 0:
                    XX = [p._vector2point(x[i]) for i in range(nXvectors)]
                    for _X in XX: 
                        _X._p = p
                        _X.dictOfFixedFuncs = p.dictOfFixedFuncs
                        _X._dictOfRedirectedFuncs = p._dictOfRedirectedFuncs
                        _X._dictOfStochVars = p._dictOfStochVars
                        _X.maxDistributionSize = p.maxDistributionSize

                r = vstack([[fun(_x) for _x in XX] if funcs[i] in p.unvectorizableFuncs else fun(X).T for i, fun in enumerate(Funcs)]).T
                
                
#                X = [p._vector2point(x[i]) for i in range(nXvectors)]
#                r = hstack([[fun(xx) for xx in X] for fun in Funcs]).reshape(1, -1)

                #new 
#                if p.vectorizable:
#                    from FuncDesigner.ooPoint import ooPoint as oopoint, multiarray
#                    
#                    X = dict([(oovar, x[:, i].view(multiarray)) for i, oovar in enumerate(p._freeVarsList)])
#                    X = oopoint(X, skipArrayCast = True)
#                    X.N = nXvectors
#                    X.isMultiPoint = True
#                    X.update(p.dictOfFixedFuncs)
#                    r = hstack([fun(X) for fun in Funcs]).reshape(1, -1)
#                
#                #old
#                else:
#                    X = [p._vector2point(x[i]) for i in range(nXvectors)]
#                    r = hstack([[fun(xx) for xx in X] for fun in Funcs]).reshape(1, -1)
            else:
                X = [(x[i],) + Args for i in range(nXvectors)] 
                
                #r = hstack([[fun(*xx) for xx in X] for fun in Funcs])
                R = []
                for xx in X:
                    tmp = [fun(*xx) for fun in Funcs]
                    r_ = hstack(tmp[0]) if len(tmp) == 1 and isinstance(tmp[0], (list, tuple)) else hstack(tmp) if len(tmp) > 1 else tmp[0]
                    R.append(r_)
                
                r = hstack(R)#.T
                #print(r.shape, userFunctionType)
                
                
        elif not getDerivative:
            tmp = [fun(*(X, )+Args) for fun in Funcs]
            r = hstack(tmp[0]) if len(tmp) == 1 and isinstance(tmp[0], (list, tuple)) else hstack(tmp) if len(tmp) > 1 else tmp[0]
                
            #print(x.shape, r.shape, x, r)
#            if not ignorePrev and ind is None:
#                p.prevVal[userFunctionType]['key'] = copy(x_0)
#                p.prevVal[userFunctionType]['val'] = r.copy()                
        elif getDerivative and p.isFDmodel:
            rr = [fun(X) for fun in Funcs]
            r = Vstack(rr) if scipyInstalled and any([isspmatrix(elem) for elem in rr]) else vstack(rr)
        else:#getDerivative

            diffInt = p.diffInt
            abs_x = abs(x)
            finiteDiffNumbers = 1e-10 * abs_x
            if p.diffInt.size == 1:
                finiteDiffNumbers[finiteDiffNumbers < diffInt] = diffInt
            else:
                finiteDiffNumbers[finiteDiffNumbers < diffInt] = diffInt[finiteDiffNumbers < diffInt]

            R = []
            #r = zeros((nFuncsToObtain, p.n))
            for index, fun in enumerate(Funcs):
                """                                                 getting derivatives                                                 """
                def func(x):
                    _r = fun(*((x,) + Args))
                    return _r if type(_r) not in (list, tuple) or len(_r)!=1 else _r[0]
                d1 = get_d1(func, x, pointVal = None, diffInt = finiteDiffNumbers, 
                            stencil=p.JacobianApproximationStencil, exactShape=True)
                #r[agregate_counter:agregate_counter+d1.size] = d1
                R.append(d1)
                    
#                agregate_counter += atleast_1d(v).shape[0]
            r = vstack(R)
            
        #if type(r) == matrix: r = r.A

        if type(r) != ndarray and not isscalar(r) and not isspmatrix(r): # multiarray
            r = r.view(ndarray).flatten() if userFunctionType == 'f' else r.view(ndarray)
        #elif userFunctionType == 'f' and p.isObjFunValueASingleNumber and prod(r.shape) > 1 and (type(r) == ndarray or min(r.shape) > 1): 
            #r = r.sum(0)
        elif userFunctionType == 'f' and p.isObjFunValueASingleNumber and not isscalar(r):
            if prod(r.shape) > 1 and not getDerivative and nXvectors == 1:
                p.err('implicit summation in objective is no longer available to prevent possibly hidden bugs')
#            if r.size == 1:
#                r = r.item()
        
        if userFunctionType == 'f' and p.isObjFunValueASingleNumber:
            if getDerivative and r.ndim > 1:
                if min(r.shape) > 1:
                    p.err('incorrect shape of objective func derivative')
                
                # TODO: omit cast to dense array. Somewhere bug triggers?
                if hasattr(r, 'toarray'):
                    r=r.toarray()
                r = r.flatten()

        if userFunctionType != 'f' and nXvectors != 1:
            r = r.reshape(nXvectors, r.size // nXvectors)
#        if type(r) == matrix: 
#            raise 0
#            r = r.A # if _dense_numpy_matrix !

        if nXvectors == 1 and (not getDerivative or prod(r.shape) == 1): # DO NOT REPLACE BY r.size - r may be sparse!
            r = r.flatten() if type(r) == ndarray else r.toarray().flatten() if not isscalar(r) else atleast_1d(r)
        
        if p.invertObjFunc and userFunctionType=='f':
            r = -r

        if not getDerivative:
            if ind is None:
                p.nEvals[userFunctionType] += nXvectors
            else:
                p.nEvals[userFunctionType] = p.nEvals[userFunctionType] + float(nXvectors * len(ind)) / getattr(p, 'n'+ userFunctionType)

        if getDerivative:
            assert x.size == p.n#TODO: add python list possibility here
            x = x_0 # for to suppress numerical instability effects while x +/- delta_x
        
        #if userFunctionType == 'f' and p.isObjFunValueASingleNumber and r.size == 1:
            #r = r.item()
        
        from FuncDesigner import _Stochastic
        if isinstance(r, _Stochastic) or (isinstance(r, ndarray) and isinstance(r.flat[0], _Stochastic)):
            p.err('''
                error in evaluation of p.%s:
                objective and constraints cannot be directly applied on stochastic variables,
                you should use functions like mean, std, var, P.''' % userFunctionType
                )
        
        if userFunctionType == 'f' and hasattr(p, 'solver') and p.solver.funcForIterFcnConnection=='f' and hasattr(p, 'f_iter') and not getDerivative:
            if p.nEvals['f']%p.f_iter == 0 or nXvectors > 1:
                p.iterfcn(x, r)

        return r




    def wrapped_1st_derivatives(p, x, ind_, funcType, ignorePrev, useSparse):
        if isinstance(x, dict):
            if not p.isFDmodel: p.err('calling the function with argument of type dict is allowed for FuncDesigner models only')
            if ind_ is not None:p.err('the operation is turned off for argument of type dict when ind!=None')
            x = p._point2vector(x)
        if ind_ is not None:
            ind = p.getCorrectInd(ind_)
        else: ind = None

        if p.istop == USER_DEMAND_EXIT:
            if p.solver.useStopByException:
                raise killThread                
            else:
                return nan
        derivativesType = 'd'+ funcType
        prevKey = p.prevVal[derivativesType]['key']
        if prevKey is not None and p.iter > 0 and np.array_equal(x, prevKey) and ind is None and not ignorePrev:
            #TODO: add counter of the situations
            assert p.prevVal[derivativesType]['val'] is not None
            return copy(p.prevVal[derivativesType]['val'])

        if ind is None and not ignorePrev: p.prevVal[derivativesType]['ind'] = copy(x)

        #TODO: patterns!
        nFuncs = getattr(p, 'n'+funcType)
        x = atleast_1d(x)
        if hasattr(p.userProvided, derivativesType) and getattr(p.userProvided, derivativesType):
               
            funcs = getattr(p.user, derivativesType)
            
            if ind is None or (nFuncs == 1 and p.functype[funcType] == 'single func'): 
                Funcs = funcs
            elif ind is not None and p.functype[funcType] == 'some funcs R^nvars -> R':
                Funcs = [funcs[i] for i in ind]
            else:
                Funcs = getFuncsAndExtractIndexes(p, funcs, ind, funcType)
            
#            if ind is None: derivativesNumber = nFuncs
#            else: derivativesNumber = len(ind)
                
            #derivatives = empty((derivativesNumber, p.n))
            derivatives = []
            #agregate_counter = 0
            for fun in Funcs:#getattr(p.user, derivativesType):
                tmp = atleast_1d(fun(*(x,)+getattr(p.args, funcType)))
                # TODO: replace tmp.size here for sparse matrices
                #assert tmp.size % p.n == mod(tmp.size, p.n)
                if tmp.size % p.n != 0:
                    if funcType=='f':
                        p.err('incorrect user-supplied (sub)gradient size of objective function')
                    elif funcType=='c':
                        p.err('incorrect user-supplied (sub)gradient size of non-lin inequality constraints')
                    elif funcType=='h':
                        p.err('incorrect user-supplied (sub)gradient size of non-lin equality constraints')
                
                if tmp.ndim == 1: m= 1
                else: m = tmp.shape[0]
                if p.functype[funcType] == 'some funcs R^nvars -> R' and m != 1:
                    # TODO: more exact check according to stored p.arr_of_indexes_* arrays
                    p.err('incorrect shape of user-supplied derivative, it should be in accordance with user-provided func size')
                derivatives.append(tmp)
                #derivatives[agregate_counter : agregate_counter + m] =  tmp#.reshape(tmp.size/p.n,p.n)
                #agregate_counter += m
            #TODO: inline ind modification!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
            
            derivatives = Vstack(derivatives)
            if ind is None:
                p.nEvals[derivativesType] += 1
            else:
                #derivatives = derivatives[ind]
                p.nEvals[derivativesType] = p.nEvals[derivativesType] + float(len(ind)) / nFuncs

            if funcType=='f':
                if p.invertObjFunc: derivatives = -derivatives
                if p.isObjFunValueASingleNumber: 
                    if not isinstance(derivatives, ndarray): derivatives = derivatives.toarray()
                    derivatives = derivatives.flatten()
        
        else:
        #if not getattr(p.userProvided, derivativesType) or p.isFDmodel:
            #                                            x, IND, userFunctionType, ignorePrev, getDerivative
            derivatives = p.wrapped_func(x, ind, funcType, True, True)
            
            if ind is None:
                p.nEvals[derivativesType] -= 1
            else:
                p.nEvals[derivativesType] = p.nEvals[derivativesType] - float(len(ind)) / nFuncs
        #else:
        
        
        
        
        if useSparse is False or not scipyInstalled or not hasattr(p, 'solver') or not p.solver._canHandleScipySparse: 
            # p can has no attr 'solver' if it is called from checkdf, checkdc, checkdh
            if not isinstance(derivatives, ndarray): 
                derivatives = derivatives.toarray()

#        if min(derivatives.shape) == 1: 
#            if isspmatrix(derivatives): derivatives = derivatives.A
#            derivatives = derivatives.flatten()
        if type(derivatives) != ndarray and isinstance(derivatives, ndarray): # dense numpy matrix
            derivatives = derivatives.A
            
        if ind is None and not ignorePrev: p.prevVal[derivativesType]['val'] = derivatives

        if funcType=='f':
            if hasattr(p, 'solver') and not p.solver.iterfcnConnected  and p.solver.funcForIterFcnConnection=='df':
                if p.df_iter is True: p.iterfcn(x)
                elif p.nEvals[derivativesType]%p.df_iter == 0: p.iterfcn(x) # call iterfcn each {p.df_iter}-th df call
            if p.isObjFunValueASingleNumber and type(derivatives) == ndarray and derivatives.ndim > 1:
                derivatives = derivatives.flatten()
        
        return derivatives


    # the funcs below are not implemented properly yet
    def user_d2f(p, x):
        assert x.ndim == 1
        p.nEvals['d2f'] += 1
        assert(len(p.user.d2f)==1)
        r = p.user.d2f[0](*(x, )+p.args.f)
        if p.invertObjFunc:# and userFunctionType=='f': 
            r = -r
        return r

    def user_d2c(p, x):
        return ()

    def user_d2h(p, x):
        return ()

    def user_l(p, x):
        return ()

    def user_dl(p, x):
        return ()

    def user_d2l(p, x):
        return ()

    def getCorrectInd(p, ind):
        if ind is None or type(ind) in [list, tuple]:
            result = ind
        else:
            try:
                result = atleast_1d(np.asarray(ind, dtype=int32)).tolist()
            except:
                raise ValueError('%s is an unknown func index type!'%type(ind))
        return result

def getFuncsAndExtractIndexes(p, funcs, ind, userFunctionType):
    if ind is None: return funcs
    if len(funcs) == 1 : 
        def f (*args, **kwargs): 
            tmp = funcs[0](*args, **kwargs)
            if isspmatrix(tmp):
                tmp = tmp.tocsc()
            elif not isinstance(tmp,  ndarray) or tmp.ndim == 0:
                tmp = atleast_1d(tmp)
            if isPyPy:
                return atleast_1d([tmp[i] for i in ind])
            else:
#                if p.debug:
#                    assert all(tmp[ind] == atleast_1d([tmp[i] for i in ind]))
                return tmp[ind]
        return [f]
    
    #getting number of block and shift
    arr_of_indexes = getattr(p, 'arr_of_indexes_' + userFunctionType)
    
    if isPyPy: # temporary walkaround the bug "int32 is unhashable"
        Left_arr_indexes = np.searchsorted(arr_of_indexes, ind) 
        left_arr_indexes = [int(elem) for elem in atleast_1d(Left_arr_indexes)]
    else:
        left_arr_indexes = np.searchsorted(arr_of_indexes, ind) 

    indLenght = len(ind)
    
    Funcs2 = []
    # TODO: try to get rid of cycles, use vectorization instead
    IndDict = {}
    for i in range(indLenght):
        
        if left_arr_indexes[i] != 0:
            num_of_funcs_before_arr_left_border = arr_of_indexes[left_arr_indexes[i]-1]
            inner_ind = ind[i] - num_of_funcs_before_arr_left_border - 1
        else:
            inner_ind = ind[i]
            
        if left_arr_indexes[i] in IndDict.keys():
            IndDict[left_arr_indexes[i]].append(inner_ind)
        else:
            IndDict[left_arr_indexes[i]] = [inner_ind]
            Funcs2.append([funcs[left_arr_indexes[i]], IndDict[left_arr_indexes[i]]])
    
    Funcs = []

    for i in range(len(Funcs2)):
        def f_aux(x, i=i): 
            r = Funcs2[i][0](x)
            # TODO: are other formats better?
            if not isscalar(r):
                if isPyPy:
                    if isspmatrix(r):
                        r = r.tocsc()[Funcs2[i][1]]
                    else:
                        # Temporary walkaround of PyPy integer indexation absence 
                        tmp = atleast_1d(r)
                        r = atleast_1d([tmp[i] for i in Funcs2[i][1]])
                else:
                    r = r.tocsc()[Funcs2[i][1]] if isspmatrix(r) else atleast_1d(r)[Funcs2[i][1]]
            return r
        Funcs.append(f_aux)
        #Funcs.append(lambda x, i=i: Funcs2[i][0](x)[Funcs2[i][1]])
    return Funcs#, inner_ind
    



from scipy.optimize import fmin_powell
from openopt.kernel.baseSolver import baseSolver
from openopt.kernel.ooMisc import isSolved

class scipy_powell(baseSolver):
    __name__ = 'scipy_powell'
    __license__ = "BSD"
    #__authors__ =
    __alg__ = ""
    __info__ = 'unconstrained NLP solver, cannot handle user-supplied gradient'
    iterfcnConnected = True

    def __init__(self):pass
    def __solver__(self, p):

        def iterfcn(x):
            p.xk, p.fk = x, p.f(x)
            p.iterfcn()
            iter = p.iter - 1
            if p.istop: raise isSolved

        try:
            iterfcn(p.x0)
            xf = fmin_powell(p.f, p.x0, xtol=p.xtol, ftol = p.ftol, disp = 0, maxiter=p.maxIter, maxfun = p.maxFunEvals, callback=iterfcn)
        except isSolved:
            xf = p.xk

        ff = p.f(p.xk)

        p.xk = p.xf = xf
        p.fk = p.ff = ff
        p.istop = 1000
        p.iterfcn()



#!/usr/bin/python
'''
Simplest OpenOpt KSP example;
requires FuncDesigner installed.
For some solvers limitations on time, cputime, "enough" value, basic GUI features are available.
See http://openopt.org/KSP for more details
'''
from openopt import *
from numpy import sin, cos

N = 150

items = [{'name': 'item %d' % i,'weight': 1.5*(cos(i)+1)**2, 
'volume': 2*sin(i) + 3, 'n':  1 if i < N/3 else 2 if i < 2*N/3 else 3} for i in range(N)]
constraints = lambda values: values['volume'] < 10

p = KSP('weight', items, constraints = constraints) 
r = p.solve('glpk', iprint = 0) # requires cvxopt and glpk installed, see http://openopt.org/KSP for other solvers
#Solver:   Time Elapsed = 0.73 	CPU Time Elapsed = 0.55
#objFunValue: 27.389749 (feasible, MaxResidual = 0)
print(r.xf) # {'item 131': 2, 'item 18': 1, 'item 62': 2, 'item 87': 1, 'item 43': 1}
# pay attention that Python indexation starts from zero: item 0, item 1 ...
# if fields 'name' are absent, you'll have list of numbers instead of Python dict



from baseProblem import NonLinProblem
from numpy import  sort, ndarray

class MINLP(NonLinProblem):
    _optionalData = ['A', 'Aeq', 'b', 'beq', 'lb', 'ub', 'c', 'h', 'discreteVars']
    probType = 'MINLP'
    allowedGoals = ['minimum', 'min', 'maximum', 'max']
    showGoal = True
    plotOnlyCurrentMinimum = True
    expectedArgs = ['f', 'x0']
    def __init__(self, *args, **kwargs):
        self.goal = 'minimum'
        self.discreteVars = {}
        NonLinProblem.__init__(self, *args, **kwargs)
        self.iprint=1

    def _Prepare(self):
        if hasattr(self, 'prepared') and self.prepared == True:
            return
        NonLinProblem._Prepare(self)    
        self.prepared = True
        if self.isFDmodel:
            r = {}
            for iv in self.freeVars:
                if iv.domain is None: continue
                ind1, ind2 = self._oovarsIndDict[iv]
                assert ind2-ind1 == 1 
                r[ind1] = iv.domain
            self.discreteVars = r
        # TODO: use something else instead of dict.keys()
        new_discr = {}
        for key, fv in self.discreteVars.items():
            if type(fv) not in [list, tuple, ndarray] and fv not in ('bool', bool):
                self.err('each element from discreteVars dictionary should be list or tuple of allowed values')
            if fv is not bool and fv is not 'bool': 
                new_discr[key] = sort(fv)
            lowest = 0 if fv is bool or fv is 'bool' else fv[0] 
            biggest = 1 if fv is bool or fv is 'bool' else fv[-1] 
            if lowest > self.ub[key]:
                self.err('variable '+ str(key)+ ': smallest allowed discrete value ' + str(fv[0]) + ' exeeds imposed upper bound '+ str(self.ub[key]))
            if biggest < self.lb[key]:
                self.err('variable '+ str(key)+ ': biggest allowed discrete value ' + str(fv[-1]) + ' is less than imposed lower bound '+ str(self.lb[key]))
        self.discreteVars.update(new_discr)
        



from baseProblem import MatrixProblem
from numpy import asfarray, ones, inf, dot, asfarray, nan, zeros, isfinite, all


class SDP(MatrixProblem):
    _optionalData = ['A', 'Aeq', 'b', 'beq', 'lb', 'ub', 'S', 'd']
    expectedArgs = ['f']
    goal = 'minimum'
    #TODO: impolement goal = max, maximum for SDP
    #allowedGoals = ['minimum', 'min', 'maximum', 'max']
    allowedGoals = ['minimum', 'min']
    showGoal = True    
    def __init__(self, *args, **kwargs):
        self.probType = 'SDP'
        self.S = {}
        self.d = {}
        MatrixProblem.__init__(self, *args, **kwargs)
        self.f = asfarray(self.f)
        self.n = self.f.size
        if self.x0 is None: self.x0 = zeros(self.n)
        
    def _Prepare(self):
        MatrixProblem._Prepare(self)
        if self.solver.__name__ in ['cvxopt_sdp', 'dsdp']:
            try:
                from cvxopt.base import matrix
                matrixConverter = lambda x: matrix(x, tc='d')
            except:
                self.err('cvxopt must be installed')
        else:
            matrixConverter = asfarray
        for i in self.S.keys(): self.S[i] = matrixConverter(self.S[i])
        for i in self.d.keys(): self.d[i] = matrixConverter(self.d[i])
#        if len(S) != len(d): self.err('semidefinite constraints S and d should have same length, got '+len(S) + ' vs '+len(d)+' instead')
#        for i in range(len(S)):
#            d[i] = matrixConverter(d[i])
#            for j in range(len(S[i])):
#                S[i][j] = matrixConverter(S[i][j])
            
        
        
    def __finalize__(self):
        MatrixProblem.__finalize__(self)
        if self.goal in ['max', 'maximum']:
            self.f = -self.f
            for fn in ['fk', ]:#not ff - it's handled in other place in RunProbSolver.py
                if hasattr(self, fn):
                    setattr(self, fn, -getattr(self, fn))
        

    def objFunc(self, x):
        return asfarray(dot(self.f, x).sum()).flatten()



from openopt import EIG

# create a 5 x 5 matrix
import numpy.random as nr
nr.seed(0)
N = 5
A = nr.rand(N, N) 

#define prob
p = EIG(A, goal = {'lm':3}) # search for 3 eigenvalues of largest magnitude
# or goal={'largest magnitude':3}, with or without space inside, case-insensitive
# for whole list of available goals see http://openopt.org/EIG

#solve
r = p.solve('arpack') # arpack is name of the involved solver

print(r.eigenvalues) # [ 0.14607289-0.19602952j -0.65372843+0.j          2.89776724+0.j        ]
# for i-th eigenvalue r.eigenvectors[:,i]  is corresponding vector, 
# as well as it is done for numpy/scipy functions
print(r.eigenvectors) 
'''
[[-0.10391145-0.56334829j  0.19592536+0.j          0.43733688+0.j        ]
 [-0.20999235+0.1812288j  -0.03219327+0.j          0.49662623+0.j        ]
 [-0.21334642+0.21648181j -0.55544796+0.j          0.42977207+0.j        ]
 [ 0.34828527+0.36295959j  0.62338178+0.j          0.38727512+0.j        ]
 [ 0.04820760-0.49714496j -0.51327338+0.j          0.47687818+0.j        ]]
 '''



class OpenOptException(BaseException):
    def __init__(self,  msg):
        self.msg = msg
    def __str__(self):
        return self.msg
        #pass

#def ooassert(cond, msg):
#    assert cond, msg

def oowarn(msg):
    s = oowarn.s + msg
    print(s)
    return s
oowarn.s = 'OpenOpt Warning: '

errSet = set()
def ooerr(msg):
    s = ooerr.s + msg
    if msg not in errSet:
        print(s)
    errSet.add(msg)
    raise OpenOptException(msg)
ooerr.s = 'OpenOpt Error: '
ooerr.set = errSet

pwSet = set()
def ooPWarn(msg):
    if msg in pwSet: return ''
    pwSet.add(msg)
    oowarn(msg)
    return msg
ooPWarn.s = 'OpenOpt Warning: '
ooPWarn.set = pwSet
    
def ooinfo(msg):
    s = ooinfo.s + msg
    print(s)
    return s
ooinfo.s = 'OpenOpt info: '

def oohint(msg):
    s = oohint.s + msg
    print(s)
    return s
oohint.s = 'OpenOpt hint: '

def oodisp(msg):
    print(msg)
    return msg
oodisp.s = ''


def oodebugmsg(p,  msg):
    if p.debug: 
        print('OpenOpt debug msg: %s' % msg)
        return msg
    return ''



import os, sys, numpy as np
from oologfcn import OpenOptException
from numpy import zeros, hstack, vstack, ndarray, copy, where, prod, isscalar, atleast_2d, eye, \
diag, asfarray

syspath = sys.path
Sep = os.sep

try:
    import scipy
    scipyInstalled = True
    scipyAbsentMsg = ''
    from scipy.sparse import isspmatrix, csr_matrix, coo_matrix
    from scipy.sparse import hstack as HstackSP, vstack as VstackSP, find as Find
    
    def Hstack(Tuple):
        ind = where([isscalar(elem) or prod(elem.shape)!=0 for elem in Tuple])[0].tolist()
        elems = [Tuple[i] for i in ind]
        if any([isspmatrix(elem) for elem in elems]):
            return HstackSP(elems)
        
        s = set([(0 if isscalar(elem) else elem.ndim) for elem in elems])
        ndim = max(s)
        if ndim <= 1:  return hstack(elems)
        #assert ndim <= 2 and 1 not in s, 'bug in OpenOpt kernel, inform developers'
        return hstack(elems) if 0 not in s else hstack([atleast_2d(elem) for elem in elems])
        
    def Vstack(Tuple):
        ind = where([prod(elem.shape)!=0 for elem in Tuple])[0].tolist()
        elems = [Tuple[i] for i in ind]
        if any([isspmatrix(elem) for elem in elems]):
            return VstackSP(elems)
        
        s = set([(0 if isscalar(elem) else elem.ndim) for elem in elems])
        s.add(0)
        ndim = max(s)
        if ndim <= 1:  return vstack(elems) if len(elems) > 0 else asfarray([])
        #assert ndim <= 2 and 1 not in s, 'bug in OpenOpt kernel, inform developers'
        return vstack(elems) if 0 not in s else vstack([atleast_2d(elem) for elem in elems])
        
    #Hstack = lambda Tuple: HstackSP(Tuple) if any([isspmatrix(elem) for elem in Tuple]) else hstack(Tuple)
    #Vstack = lambda Tuple: VstackSP(Tuple) if any([isspmatrix(elem) for elem in Tuple]) else vstack(Tuple)
    SparseMatrixConstructor = lambda *args, **kwargs: scipy.sparse.lil_matrix(*args, **kwargs)
except:
    scipyInstalled = False
    csr_matrix = None
    coo_matrix = None
    scipyAbsentMsg = 'Probably scipy installation could speed up running the code involved'
    isspmatrix = lambda *args,  **kwargs:  False
    Hstack = hstack
    Vstack = vstack
    def SparseMatrixConstructor(*args, **kwargs): 
        raise OpenOptException('error in OpenOpt kernel, inform developers')
    def Find(*args, **kwargs): 
        raise OpenOptException('error in OpenOpt kernel, inform developers')


# temporary, until normal pypy implementation
def PyPy_where(*args):
    if len(args) == 3:
        return np.where(*args)
    elif len(args) != 1:
        assert 0, 'number of arguments for numpy.where must be 1 or 3'
    A = np.asarray(args[0])
    if A.ndim > 2:
        assert 0, 'unimplemented yet'
    if A.ndim == 0:
        return (np.array([0]),) if A != 0 else (np.array([], dtype=np.int32),)
    if A.ndim == 1:
        r = (np.array([i for i, elem in enumerate(A) if elem != 0.0]),)
        return r
        
    # A.ndim == 2
    I, J = [], []
    for i in range(A.shape[0]):
        for j in range(A.shape[1]):
            if A[i, j] != 0.0:
                I.append(i)
                J.append(j)
    return (np.array(I), np.array(J))


try:
    import __pypy__
    isPyPy = True
    where = PyPy_where
except ImportError:
    isPyPy = False

DenseMatrixConstructor = lambda *args, **kwargs: zeros(*args, **kwargs)

pwSet = set()
def pWarn(msg):
    if msg in pwSet: return
    pwSet.add(msg)
    print('Warning: ' + msg)

def Eye(n):
    if not scipyInstalled and n>150:
        pWarn(scipyAbsentMsg)
    if n == 1:
        return 1.0
    elif n <= 16 or not scipyInstalled:
        return eye(n)
    else:
        return scipy.sparse.identity(n)

def Diag(x):
    if not scipyInstalled and len(x)>150: 
        pWarn(scipyAbsentMsg)
    if isscalar(x): return x
    elif len(x) == 1: return asfarray(x)
    elif len(x) < 16 or not scipyInstalled: return diag(x)
    else: return scipy.sparse.spdiags(x, [0], len(x), len(x)) 


##################################################################
solverPaths = {}
from os import path as os_path
FILE = os_path.realpath(__file__)
for root, dirs, files in os.walk(''.join([elem + os.sep for elem in FILE.split(os.sep)[:-2]+ ['solvers']])):
    rd = root.split(os.sep)
    if '.svn' in rd or '__pycache__' in rd: continue
    rd = rd[rd.index('solvers')+1:]
    for file in files:
        if file.endswith('_oo.py'):
            solverPaths[file[:-6]] = ''.join(rd+['.',file[:-3]])


def getSolverFromStringName(p, solver_str):
    if solver_str not in solverPaths:
        p.err('''
        incorrect solver is called, maybe the solver "%s" is misspelled 
        or requires special installation and is not installed, 
        check http://openopt.org/%s''' % (solver_str, p.probType))
    if p.debug:
        solverClass =  solver_import(solverPaths[solver_str], solver_str)
    else:
        try:
            solverClass = solver_import(solverPaths[solver_str], solver_str)
        except ImportError:
            p.err('incorrect solver is called, maybe the solver "' + solver_str +'" require its installation, check http://www.openopt.org/%s or try p._solve() for more details' % p.probType)
    r = solverClass()        
    if not hasattr(r, 'fieldsForProbInstance'): r.fieldsForProbInstance = {}
    return r

##################################################################
importedSet = set()
ooPath = ''.join(elem+Sep for elem in __file__.split(Sep)[:-3])
def solver_import(solverPath, solverName):
    if solverPath not in importedSet:
        importedSet.add(solverPath)
        syspath.append(ooPath+'openopt'+Sep + 'solvers'+''.join(Sep+elem for elem in solverPath.split('.')[:-1]))
    name = 'openopt.solvers.' + solverPath
    mod = __import__(name)
    components = name.split('.')
    for comp in components[1:]:
        mod = getattr(mod, comp)
    return getattr(mod, solverName)

def oosolver(solverName, *args,  **kwargs):
    if args != ():
        raise OpenOptException("Error: oosolver() doesn't consume any *args, use **kwargs only")
    from openopt.kernel.baseSolver import baseSolver
    if isinstance(solverName, baseSolver):
        return solverName
    try:
        if ':' in solverName:
            # TODO: make it more properly
            # currently it's used for to get filed isInstalled value
            # from ooSystem
            solverName = solverName.split(':')[1]
        solverClass = solver_import(solverPaths[solverName], solverName)
        solverClassInstance = solverClass()
        solverClassInstance.fieldsForProbInstance = {}
        for key, value in kwargs.items():
            if hasattr(solverClassInstance, key):
                setattr(solverClassInstance, key, value)
            else:
                solverClassInstance.fieldsForProbInstance[key] = value
        solverClassInstance.isInstalled = True
    except ImportError:
        
        solverClassInstance = baseSolver()
        solverClassInstance.__name__ = solverName
        solverClassInstance.fieldsForProbInstance = {}
        solverClassInstance.isInstalled = False
    #assert hasattr(solverClassInstance, 'fieldsForProbInstance')
    return solverClassInstance

def Copy(arg): 
    return arg.copy() if isinstance(arg, ndarray) or isspmatrix(arg) else copy(arg)

class EmptyClass: pass



from openopt.kernel.baseSolver import baseSolver
from openopt.kernel.nonOptMisc import Vstack
from scipy.sparse.linalg import eigs

#from nonOptMisc import Hstack

class arpack(baseSolver):
    __name__ = 'arpack'
    __license__ = "BSD"
    __authors__ = ''
    __alg__ = ''
    __info__ = """    """

    __optionalDataThatCanBeHandled__ = ['M']
    _canHandleScipySparse = True
    
    #def __init__(self): pass

    def __solver__(self, p):
        A = p.C
        M = p.M
        
        if p._goal == 'all':
            p.err('You should change prob "goal" argument, solver arpack can search at most n-2 eigenvectors')
        
        if p.N > A.shape[0] - 2:
            p.err('solver arpack can find at most n-2 eigenvalues, where n is height of matrix')
            
        eigenvalues, eigenvectors = \
        eigs(A, k=p.N, M=M, sigma=None, which='LM', v0=None, ncv=None, maxiter=None, \
             tol=p.xtol, return_eigenvectors=True)
        p.xf = p.xk = Vstack((eigenvalues, eigenvectors))
        p.eigenvalues = eigenvalues
        p.eigenvectors = eigenvectors
        p.ff = 0



#!/usr/bin/python
'''
A simple OpenOpt TSP example for directed graph;
requires networkx (http://networkx.lanl.gov)
and FuncDesigner installed.
For some solvers limitations on time, cputime, "enough" value, basic GUI features are available.
See http://openopt.org/TSP for more details
'''
from openopt import *
from numpy import sin, cos
import networkx as nx

N = 5
G = nx.DiGraph()
G.add_edges_from(\
                 [('node %d' % i, 'node %d' % j,{'time': 1.5*(cos(i)+sin(j)+1)**2, 'cost':(i-j)**2 + 2*sin(i) + 2*cos(j)+1}) for i in range(N) for j in range(N) if i != j ])


# for solver sa handling of constraints is unimplemented yet
# when your solver is interalg you can use nonlinear constraints as well
# for nonlinear objective and constraints functions like arctan, abs etc should be imported from FuncDesigner
constraints = lambda values: (2 * values['time'] + 3 * values['cost'] > 100, 8 * values['time'] + 15*values['cost'] > 150)
objective = lambda values: values['time'] + 10*values['cost'] 
# objective and constraint funcs are applied on sums of all values for all involved edges, not for each standalone edge

p = TSP(G, objective = objective, constraints = constraints, start = 'node 3', returnToStart=False)

r = p.solve('lpSolve') # also you can use some other solvers - sa, interalg, OpenOpt MILP solvers

# if your solver is cplex or interalg, you can provide some stop criterion, 
# e.g. maxTime, maxCPUTime, fEnough etc, for example 
# r = p.solve('cplex', maxTime = 100, fEnough = 10) 
# i.e. stop if solution with 10 nodes has been obtained or 100 sec were elapsed
# (gplk and lpSolve has no iterfcn connected and thus cannot handle those criterions)

# also you can use p.manage() to enable basic GUI (http://openopt.org/OOFrameworkDoc#Solving) 
# it requires tkinter installed, that is included into PythonXY, EPD;
# for linux use [sudo] easy_install tk or [sodo] apt-get install python-tk
#r = p.manage('cplex')

print(r.nodes)
print(r.edges) # (node_from, node_to)
print(r.Edges) # full info on edges; unavailable for solver sa yet
'''
Solver:   Time Elapsed = 0.04 	CPU Time Elapsed = 0.04
# for N = 13 (full-13 graph) lpSolve Time Elapsed = 1.76 	CPU Time Elapsed = 1.74
objFunValue: 231.72217 (feasible, MaxResidual = 0)
['node 3', 'node 4', 'node 1', 'node 0', 'node 2']
[('node 3', 'node 4'), ('node 4', 'node 1'), ('node 1', 'node 0'), ('node 0', 'node 2'), ('node 2', 'node 3')]
[('node 3', 'node 4', {'cost': 0.9749527743925106, 'time': 0.83655413990914185}), 
('node 4', 'node 1', {'cost': 9.5669996211204236, 'time': 2.1164007698022411}), 
('node 1', 'node 0', {'cost': 5.6829419696157935, 'time': 3.5587967901940627}), 
('node 0', 'node 2', {'cost': 4.1677063269057157, 'time': 12.6960172766018}), 
('node 2', 'node 3', {'cost': 1.8386098604504726, 'time': 0.78837914911982798})]
'''



from openopt import NLP
from numpy import cos, arange, ones, asarray, zeros, mat, array
N = 100

# objective function:
f = lambda x: ((x-5)**2).sum()

# objective function gradient (optional):
df = lambda x: 2*(x-5)

# start point (initial estimation)
x0 = 8*cos(arange(N))

# c(x) <= 0 constraints
def c(x):
    r = zeros(N-1)
    for i in xrange(N-1):
        r[i] = x[i]**2 + x[i+1]**2 - 1
    return r

def dc(x):
    r = zeros((N-1, N))
    for i in xrange(N-1):
        r[i, i] = 2*x[i]
        r[i, i+1] = 2*x[i+1]
    return r

#    counter = 0
#    r = zeros(N*(N-1)/2)
#    for i in xrange(N):
#        for j in xrange(i):
#            r[counter] = x[i]**2 + x[j]**2 - 1
#            counter += 1
#    return r

lb = -6*ones(N)
ub = 6*ones(N)
ub[4] = 4
lb[5], ub[5] = 8, 15

# general linear inequality constraints
#A = zeros((3, N))
#A[0, 9] = 1
#A[0, 19] = 1
#A[1, 10:12] = 1
#A[2] = -ones(N)
#b = [7, 9, -825]
A, b=None, None


# general linear equality constraints
#Aeq = zeros(N)
#Aeq[10:12] = 1
#beq = 11
Aeq, beq = None, None

# required constraints tolerance, default for NLP is 1e-6
contol = 1e-7

# If you use solver algencan, NB! - it ignores xtol and ftol; using maxTime, maxCPUTime, maxIter, maxFunEvals, fEnough is recommended.
# Note that in algencan gtol means norm of projected gradient of  the Augmented Lagrangian
# so it should be something like 1e-3...1e-5
gtol = 1e-7 # (default gtol = 1e-6)

# Assign problem:
# 1st arg - objective function
# 2nd arg - start point

p = NLP(f, x0, df=df,  c=c, dc=dc,   
        gtol=gtol, contol=contol, iprint = 50, maxIter = 10000, maxFunEvals = 1e7, name = 'NLP_1')

#p = NLP(f, x0, df=df,  Aeq=Aeq, beq=beq, 
#        gtol=gtol, contol=contol, iprint = 50, maxIter = 10000, maxFunEvals = 1e7, name = 'NLP_1')

#p = NLP(f, x0, df=df,  lb=lb, ub=ub, gtol=gtol, contol=contol, iprint = 50, maxIter = 10000, maxFunEvals = 1e7, name = 'NLP_1')

#optional: graphic output, requires pylab (matplotlib)
p.plot = 0

#optional: user-supplied 1st derivatives check
p.checkdf()
p.checkdc()
p.checkdh()


def MyIterFcn(p):
    return 0
#    print 'Iteration',p.iter
#    if p.iter == 50:
#       p.user.mylist.append(p.xk.copy()) 
#    return 0

p.user.mylist = []
# solve the problem
#p.debug=1
solver = 'algencan'
solver = 'ralg'
#solver = 'ipopt'
#solver = 'scipy_slsqp'

p.debug=1
r = p.solve(solver, showLS=0, iprint=10, maxTime = 50,  callback = MyIterFcn) # string argument is solver name
#r = p.solve('r2', iprint = 1, plot=0, showLS=1, maxIter=480)

# r.xf and r.ff are optim point and optim objFun value
# r.ff should be something like 132.0522 



from openopt.kernel.baseSolver import baseSolver
from CVXOPT_QP_Solver import CVXOPT_QP_Solver

class cvxopt_qp(baseSolver):
    __name__ = 'cvxopt_qp'
    __license__ = "LGPL"
    __authors__ = "http://abel.ee.ucla.edu/cvxopt"
    __alg__ = "see http://abel.ee.ucla.edu/cvxopt"
    __optionalDataThatCanBeHandled__ = ['A', 'Aeq', 'b', 'beq', 'lb', 'ub']
    properTextOutput = True
    _canHandleScipySparse = True
    def __init__(self): pass
    def __solver__(self, p):
        return CVXOPT_QP_Solver(p, 'native_CVXOPT_QP_Solver')



from openopt.kernel.baseSolver import baseSolver
from openopt import OpenOptException

class glpk(baseSolver):
    __name__ = 'glpk'
    __license__ = "GPL v.2"
    __authors__ = "http://www.gnu.org/software/glpk + Python bindings from http://abel.ee.ucla.edu/cvxopt"
    __homepage__ = 'http://www.gnu.org/software/glpk'
    #__alg__ = ""
    __optionalDataThatCanBeHandled__ = ['A', 'Aeq', 'b', 'beq', 'lb', 'ub', 'intVars', 'binVars']
    _canHandleScipySparse = True
    
    def __init__(self): 
        try:
            import cvxopt
        except ImportError:
            raise OpenOptException('for solver glpk cvxopt is required, but it was not found')

    def __solver__(self, p):
        from CVXOPT_LP_Solver import CVXOPT_LP_Solver
        return CVXOPT_LP_Solver(p, 'glpk')



from numpy import dot, diag, ones, zeros, sqrt
from openopt.kernel.ooMisc import norm

def amsg2p(f, df, x0, epsilon, f_opt, gamma, callback = lambda x, f: False):
    # returns optim point and iteration number
    f0 = f(x0)
    if f0 - f_opt <= epsilon: return x0, 0
    x, n = x0.copy(), x0.size
    df0 = df(x0)
    ndf = norm(df0)
    h, dzeta, p, B = gamma * (f0 - f_opt) / ndf, df0 / ndf, zeros(n), diag(ones(n, 'float64')) # TODO: add possibility to create B of type float128
    k = 0
    while True:
        k += 1
        x -= h * dot(B, dzeta)
        F = f(x) 
        r = callback(x, F) 
        if r not in (0, False, None): 
            break # user-demanded stop
        if F - f_opt <= epsilon: break
        DF = df(x)
        DF_dilated = dot(B.T, DF)
        nDF_dilated = norm(DF_dilated)
        dzeta_new, h = DF_dilated / nDF_dilated, gamma * (F-f_opt) / nDF_dilated
        lambda1, lambda2 = -dot(p, dzeta_new), -dot(dzeta, dzeta_new)
        c1, c2 = lambda1>0, lambda2>0
        p = (lambda1 * p + lambda2 * dzeta)/sqrt(lambda1**2+lambda2**2) if c1 and c2 else dzeta if c2 and not c1 else zeros(n) if not c1 and not c2 else p
        mu = dot(p, dzeta_new)
        if -1 < mu < 0:
            S = sqrt(1-mu**2)
            nu = (1/S-1) * dzeta_new - (mu/S) * p 
            B += dot(dot(B, nu.reshape(n, 1)), dzeta_new.reshape(1, n))
            h /= S
            p = (p - mu * dzeta_new) / S
        else:
            p = zeros(n)
        dzeta = dzeta_new
    return x, k



from numpy import *
from openopt import *

def pointProjection(x,  lb, ub, A, b, Aeq, beq):
    # projection of x to set of linear constraints
    n = x.size
    # TODO: INVOLVE SPARSE CVXOPT MATRICES
    p = QP(H = eye(n), f = -x, A = A, b=b, Aeq=Aeq, beq=beq, lb=lb, ub=ub)
    r = p.solve('cvxopt_qp')
    return r.xf




if __name__ == '__main__':
    x = array((1, 2, 3))
    lb, ub = None, None
    A = [3, 4, 5]
    b = -15
    Aeq, beq = None, None
    proj = pointProjection(x,  lb, ub, A, b, Aeq, beq)
    print(proj)



# created by Dmitrey
from numpy import copy, isnan, array, argmax, abs, zeros, any, isfinite, all, asscalar, \
dot, sqrt, nanmax, logical_or, prod, arange, ndarray, asarray, sum#where
from ooMisc import norm
from nonOptMisc import Copy, isPyPy, where
#from pointProjection import pointProjection
empty_arr = array(())
try:
    from scipy.sparse import isspmatrix, csr_matrix
    scipyInstalled = True
except ImportError:
    scipyInstalled = False
    isspmatrix = lambda *args,  **kwargs:  False

class Point:
    """
    the class is used to prevent calling non-linear constraints more than once
    f, c, h are funcs for obtaining objFunc, non-lin ineq and eq constraints.
    df, dc, dh are funcs for obtaining 1st derivatives.
    """
    __expectedArgs__ = ['x', 'f', 'mr']
    def __init__(self, p, x, *args, **kwargs):
        self.p = p
        self.x = copy(x)
        self.isMultiArray = self.x.ndim > 1
        for i, arg in enumerate(args):
            setattr(self, '_' + self.__expectedArgs__[i], args[i])
        for name, val in kwargs.items():
            setattr(self, '_' + name, val)
        #assert self.x is not None

    def f(self):#, _linePointDescriptor = None):
        if not hasattr(self, '_f'): 
            # TODO: rework this: self.p.probType!='IP'
            if self.p._baseClassName == 'NonLin' and self.p.probType!='IP':
                func = self.p.f if self.p.isObjFunValueASingleNumber else self.p.F
                self._f = func(self.x)#, _linePointDescriptor = _linePointDescriptor)
            else:
                self._f = self.p.objFunc(self.x)
        return copy(self._f)
        

    def df(self):
        if not hasattr(self, '_df'): self._df = self.p.df(self.x)
        return Copy(self._df)

    def c(self, ind=None):
        if not self.p.userProvided.c: 
            return empty_arr.copy()
        if ind is None:
            if not hasattr(self, '_c'): 
                self._c = self.p.c(self.x)
                self._nNaNs_C = isnan(self._c).sum(self._c.ndim-1)
                if self.isMultiArray:
                    tmp = self._c
                    delattr(self, '_c')
                    return tmp
            return copy(self._c)
        else:
            if hasattr(self, '_c'): return copy(self._c[ind])
            else: return copy(self.p.c(self.x, ind))


    def dc(self, ind=None):
        if not self.p.userProvided.c: return empty_arr.copy().reshape(0, self.p.n)
        if ind is None:
            if not hasattr(self, '_dc'): self._dc = self.p.dc(self.x)
            return Copy(self._dc)
        else:
            if hasattr(self, '_dc'): return Copy(self._dc[ind])
            else: return Copy(self.p.dc(self.x, ind))


    def h(self, ind=None):
        if not self.p.userProvided.h: return empty_arr.copy()
        if ind is None:
            if not hasattr(self, '_h'): 
                self._h = self.p.h(self.x)
                self._nNaNs_H = isnan(self._h).sum(self._h.ndim-1)
                if self.isMultiArray:
                    tmp = self._h
                    delattr(self, '_h')
                    return tmp
            return copy(self._h)
        else:
            if hasattr(self, '_h'): return copy(self._h[ind])
            else: return copy(self.p.h(self.x, ind))

    def dh(self, ind=None):
        if not self.p.userProvided.h: return empty_arr.copy().reshape(0, self.p.n)
        if ind is None:
            if not hasattr(self, '_dh'): self._dh = self.p.dh(self.x)
            return Copy(self._dh)
        else:
            if hasattr(self, '_dh'): 
                return Copy(self._dh[ind])
            else: 
                return Copy(self.p.dh(self.x, ind))

    def d2f(self):
        if not hasattr(self, '_d2f'): self._d2f = self.p.d2f(self.x)
        return copy(self._d2f)

#    def intConstraints(self):
#        if self.p.intVars == {} or self.p.intVars == []: return 0
#        if not hasattr(self, '_intConstraint'):
#            r = [norm(self.x[k]-self.p.intVars[k], -inf) for k in self.p.intVars.keys()]
#            self._intConstraints = max(r)
#
#        return copy(self._intConstraints)

    def lin_ineq(self):
        if not hasattr(self, '_lin_ineq'): self._lin_ineq = self.p._get_AX_Less_B_residuals(self.x)
        return copy(self._lin_ineq)

    def lin_eq(self):
        if not hasattr(self, '_lin_eq'): self._lin_eq = self.p._get_AeqX_eq_Beq_residuals(self.x)
        return copy(self._lin_eq)

    def lb(self):
        if not hasattr(self, '_lb'): self._lb = self.p.lb - self.x
        return copy(self._lb)

    def ub(self):
        if not hasattr(self, '_ub'): self._ub = self.x - self.p.ub
        return copy(self._ub)

    def mr(self, retAll = False, checkBoxBounds = True):
        # returns max residual
        
        if not hasattr(self, '_mr'):
            r, fname, ind = 0, None, 0
            if self.isMultiArray: 
                r = zeros(self.x.shape[0])
            ineqs = ['lin_ineq', 'lb', 'ub'] if checkBoxBounds else ['lin_ineq']
            eqs = ['lin_eq']
            if self.p._baseClassName == 'NonLin':
                ineqs.append('c')
                eqs.append('h')
#            elif self.p.probType in ['MILP', 'MINLP']:
#                pass
                #ineqs.append('intConstraints')
            for field in ineqs:
                fv = array(getattr(self, field)())#.flatten()
                if fv.size > 0 and not all(isnan(fv)):
                    val_max = nanmax(fv, fv.ndim-1)
                    r = where(r < val_max, val_max, r) # handles nan ok
                    if not self.isMultiArray:
                        ind_max = where(fv==val_max)[0][0]
                        if r < val_max:
                            r, ind, fname = val_max, ind_max, field
            for field in eqs:
                fv = array(getattr(self, field)())#.flatten()
                if fv.size > 0 and not all(isnan(fv)):
                    fv = abs(fv)
                    val_max = nanmax(fv, fv.ndim-1)
                    r = where(r < val_max, val_max, r) # handles nan ok
                    if not self.isMultiArray:
                        ind_max = argmax(fv)
                        #val_max = fv[ind_max]
                        if r < val_max:
                            r, ind, fname = val_max, ind_max, field
            if not self.isMultiArray:
                self._mr, self._mrName,  self._mrInd= r, fname, ind
        if retAll:
            return asscalar(copy(self._mr)), self._mrName, asscalar(copy(self._mrInd))
        else: return asscalar(copy(self._mr)) if not self.isMultiArray else r

    def sum_of_all_active_constraints(self):
        if not hasattr(self, '_sum_of_all_active_constraints'):
            p = self.p
            if p.solver.__name__ == 'ralg':
                tol = p.contol / 2.0
            else:
                tol = 0.0
#            elif p.solver.__name__ == 'gsubg':
#                tol = 0.0
#            else:
#                p.err('unimplemented case in Point.py')
                
            c, h= self.c(), self.h()
            all_lin = self.all_lin()
            self._sum_of_all_active_constraints = (c[c>0] - 0).sum() + (h[h>tol] - tol).sum() - (h[h<-tol] + tol).sum() + all_lin
        return Copy(self._sum_of_all_active_constraints)
                
    def mr_alt(self, retAll = False, bestFeasiblePoint=None):

        # TODO: add fix wrt bestFeasiblePoint handling
        
        ###################################################
        # DEBUG!
        # IT SAVES DIFFERENT VALUES WRT WITH OR WITHOUT bestFeasiblePoint
        # IT HAS BEEN CALLED 1ST TIME
        if hasattr(self, '_mr_alt'): delattr(self, '_mr_alt')
        ###################################################
        
        if not hasattr(self, '_mr_alt'):
            p = self.p
            if not hasattr(p.solver, 'approach') or p.solver.approach == 'all active':
                val = self.sum_of_all_active_constraints()
                if bestFeasiblePoint is not None:
                    
                    # not "+="!!!!! Else some problems with array shapes can occur
                    #print self.f()-bestFeasiblePoint.f()
#                    val = val + max((0, self.f()-bestFeasiblePoint.f())) * p.contol / p.fTol 
                    pass

                self._mr_alt, self._mrName_alt,  self._mrInd_alt = val, 'all active', 0
            else:
                p.err('bug in openopt kernel, inform developers')
#                r = 0.0
#                if c.size != 0:
#                    ind_max = argmax(c)
#                    val_max = c[ind_max]
#                    if val_max > r:
#                        r = val_max
#                        Type = 'c'
#                        ind = ind_max
#                if h.size != 0:
#                    h = abs(h)
#                    ind_max = argmax(h)
#                    val_max = h[ind_max]
#                    #hm = abs(h).max()
#                    if val_max > r:
#                        r = val_max
#                        Type = 'h'
#                        ind = ind_max                
#                lin_eq = self.lin_eq()
#                if lin_eq.size != 0:
#                    l_eq = abs(lin_eq)
#                    ind_max = argmax(l_eq)
#                    val_max = l_eq[ind_max]
#                    if val_max > r:
#                        r = val_max
#                        Type = 'lin_eq'
#                        ind = ind_max
#                lin_ineq = self.lin_ineq()
#                # TODO: implement it
#                val = r
#                self._mr_alt, self._mrName_alt,  self._mrInd_alt = val, Type, 0

        if retAll:
            return asscalar(copy(self._mr_alt)), self._mrName_alt, asscalar(copy(self._mrInd_alt))
        else: return asscalar(copy(self._mr_alt))

    def dmr(self, retAll = False):
        # returns direction for max residual decrease
        #( gradient for equality < 0 residuals ! )
        if not hasattr(self, '_dmr') or (retAll and not hasattr(self, '_dmrInd')):
            g = zeros(self.p.n)
            maxResidual, resType, ind = self.mr(retAll=True)
            if resType == 'lb':
                g[ind] -= 1 # N * (-1), -1 = dConstr/dx = d(lb-x)/dx
            elif resType == 'ub':
                g[ind] += 1 # N * (+1), +1 = dConstr/dx = d(x-ub)/dx
            elif resType == 'lin_ineq':
                g += self.p.A[ind]
            elif resType == 'lin_eq':
                rr = self.p.matmult(self.p.Aeq[ind], self.x)-self.p.beq[ind]
                if rr < 0:  g -= self.p.Aeq[ind]
                else:  g += self.p.Aeq[ind]
            elif resType == 'c':
                dc = self.dc(ind=ind).flatten()
                g += dc
            elif resType == 'h':
                dh = self.dh(ind=ind).flatten()
                if self.p.h(self.x, ind) < 0:  g -= dh#CHECKME!!
                else: g += dh#CHECKME!!
            else:
                # TODO: error or debug warning
                pass
                #self.p.err('incorrect resType')

            self._dmr, self._dmrName,  self._dmrInd = g, resType, ind
        if retAll:
            return copy(self._dmr),  self._dmrName,  copy(self._dmrInd)
        else:
            return copy(self._dmr)

    def betterThan(self, point2compare, altLinInEq = False, bestFeasiblePoint = None):
        """
        usage: result = involvedPoint.better(pointToCompare)

        returns True if the involvedPoint is better than pointToCompare
        and False otherwise
        (if NOT better, mb same fval and same residuals or residuals less than desired contol)
        """
        if self.p.isUC:
            return self.f() < point2compare.f()

        #contol = self.p.contol

        if altLinInEq:
            mr, point2compareResidual = self.mr_alt(bestFeasiblePoint=bestFeasiblePoint), point2compare.mr_alt(bestFeasiblePoint=bestFeasiblePoint)
        else:
            mr, point2compareResidual =  self.mr(), point2compare.mr()
        
#        if altLinInEq and bestFeasiblePoint is not None and isfinite(self.f()) and isfinite(point2compare.f()):
#            fTol = self.p.fTol
#            mr += (self.f()  - bestFeasiblePoint.f() + fTol) *contol / fTol
#            point2compareResidual += (point2compare.f() - bestFeasiblePoint.f()+fTol) *contol / fTol
#            mr += max((0, self.f()  - bestFeasiblePoint.f())) *contol/ fTol
#            point2compareResidual += max((0, point2compare.f() - bestFeasiblePoint.f())) *contol/ fTol
#            assert self.f() >= bestFeasiblePoint.f()
#            assert point2compare.f() >= bestFeasiblePoint.f()
#            mr += (self.f()  - bestFeasiblePoint.f()) / fTol
#            point2compareResidual += (point2compare.f() - bestFeasiblePoint.f()) / fTol
        #criticalResidualValue = max((contol, point2compareResidual))
        self_nNaNs, point2compare_nNaNs = self.nNaNs(), point2compare.nNaNs()

        if point2compare_nNaNs  > self_nNaNs: return True
        elif point2compare_nNaNs  < self_nNaNs: return False
        
        # TODO: check me
        if self_nNaNs == 0:
            if mr > self.p.contol and mr > point2compareResidual: return False
            elif point2compareResidual > self.p.contol and point2compareResidual > mr: return True
        else: # got here means self_nNaNs = point2compare_nNaNs but not equal to 0
            if mr == 0 and point2compareResidual == 0: 
                if self.p.solver.__name__ not in ('interalg', 'de'):
                    self.p.err('you should provide at least one active constraint in each point from R^n where some constraints are undefined')
            return mr < point2compareResidual

        point2compareF_is_NaN = isnan(point2compare.f())
        selfF_is_NaN = isnan(self.f())
        
        if isPyPy:
            if type(point2compareF_is_NaN) == ndarray: point2compareF_is_NaN = asscalar(point2compareF_is_NaN)
            if type(selfF_is_NaN) == ndarray: selfF_is_NaN = asscalar(selfF_is_NaN)
        
        if not point2compareF_is_NaN: # f(point2compare) is not NaN
            if not selfF_is_NaN: # f(newPoint) is not NaN
                return self.f() < point2compare.f()
            else: # f(newPoint) is NaN
                return False
        else: # f(point2compare) is NaN
            if selfF_is_NaN: # f(newPoint) is NaN
                return mr < point2compareResidual
            else: # f(newPoint) is not NaN
                return True

    def isFeas(self, altLinInEq):
        if not all(isfinite(self.f())): return False
        if self.p.isUC: return True
        if self.nNaNs() != 0: return False
        contol = self.p.contol 
        if altLinInEq:
            if  self.mr_alt() > contol: return False
        else:
            if hasattr(self, '_mr'):
                if self._mr > contol: return False
            else:
                #TODO: simplify it!
                #for fn in residuals: (...)
                if any(self.lb() > contol): return False
                if any(self.ub() > contol): return False
                if any(abs(self.lin_eq()) > contol): return False
                if any(self.lin_ineq() > contol): return False
                if any(abs(self.h()) > contol): return False
                if any(self.c() > contol): return False
        return True

    def nNaNs(self):
        # returns number of nans in constraints
        if self.p._baseClassName != 'NonLin': 
            return 0
        hasC, hasH = self.p.userProvided.c, self.p.userProvided.h
        if hasH and not hasattr(self, '_nNaNs_H'):
            self.h()
        if hasC and not hasattr(self, '_nNaNs_C'):
            self.c()
        r = (self._nNaNs_C if hasC else 0) + (self._nNaNs_H if hasH else 0)
        #r = len(where(isnan(c))[0])
        return r
    
    def linePoint(self, alp, point2, ls=None):
        # returns alp * point1 + (1-alp) * point2
        # where point1 is self, alp is real number
        #assert isscalar(alp)
        p = self.p
        r = p.point(self.x * (1-alp) + point2.x * alp)
        
        #lin_eqs = self.lin_eq()*alp +  point2.lin_eq() * (1-alp)
        #print '!>>, ',  p.norm(lin_eqs), p.norm(lin_eqs - r.lin_eq())
        
        # TODO: optimize it, take ls into account!
        #if ls is not None and 
        if not (p.iter % 16):
            lin_ineq_predict = self.lin_ineq()*(1-alp) +  point2.lin_ineq() * alp
            #if 1 or p.debug: print('!>>', p.norm(lin_ineq_predict-r.lin_ineq()))
            r._lin_ineq = lin_ineq_predict
            r._lin_eq = self.lin_eq()*(1-alp) +  point2.lin_eq() * alp
        
        # don't calculate c for inside points
        if 0<alp<1:
            c1, c2 = self.c(), point2.c()
            ind1 = logical_or(c1 > 0,  isnan(c1))
            ind2 = logical_or(c2 > 0,  isnan(c2))
            
            # prev
#            ind = where(ind1 | ind2)[0]
#            
#            _c = zeros(p.nc)
#            if ind.size != 0:
#                _c[ind] = p.c(r.x, ind)

            # new, temporary walkaround for PyPy
            ind = logical_or(ind1, ind2)
            
            _c = zeros(p.nc)
            if any(ind):
                _c[ind] = p.c(r.x, where(ind)[0])
            r._c = _c
            r._nNaNs_C = isnan(_c).sum(_c.ndim-1)
        
        # TODO: mb same for h?
        
        #r._linePointDescriptor = alp # (self, alp, point2)
        return r


    def all_lin(self): # TODO: rename it wrt lin_eq that are present here
        if not hasattr(self, '_all_lin'):
            lb, ub, lin_ineq = self.lb(), self.ub(), self.lin_ineq()
            r = 0.0
            # TODO: CHECK IT - when 0 (if some nans), when contol

#            if all(isfinite(self.f())): threshold = self.p.contol
#            else: 0.0 = 0
            lin_eq = self.lin_eq()
            ind_lb, ind_ub = lb>0.0, ub>0.0
            ind_lin_ineq = lin_ineq>0.0
            ind_lin_eq = abs(lin_eq)>0.0
            USE_SQUARES = 1
            if USE_SQUARES:
                if any(ind_lb):
                    r += sum(lb[ind_lb] ** 2)
                if any(ind_ub):
                    r += sum(ub[ind_ub] ** 2)

#                if ind_lb.size != 0:
#                    r += sum(lb[ind_lb])
#                if ind_ub.size != 0:
#                    r += sum(ub[ind_ub])
                if any(ind_lin_ineq):
                    r += sum(lin_ineq[ind_lin_ineq] ** 2)
                if any(ind_lin_eq):
                    r += sum(lin_eq[ind_lin_eq] ** 2)

                self._all_lin = r / self.p.contol
#                self._all_lin = r
            else:
                if any(ind_lb):
                    r += sum(lb[ind_lb])
                if any(ind_ub):
                    r += sum(ub[ind_ub])
                if any(ind_lin_ineq):
                    r += sum(lin_ineq[ind_lin_ineq])
                if any(ind_lin_eq):
                    r += sum(abs(lin_eq[ind_lin_eq]))
                self._all_lin = r
                    
        return copy(self._all_lin)

    def all_lin_gradient(self):
        if not hasattr(self, '_all_lin_gradient'):
            p = self.p
            n = p.n
            d = zeros(n)


            lb, ub = self.lb(), self.ub()
            lin_ineq = self.lin_ineq()
            lin_eq = self.lin_eq()
            ind_lb, ind_ub = lb > 0.0, ub > 0.0
            ind_lin_ineq = lin_ineq > 0.0
            ind_lin_eq = abs(lin_eq) != 0.0
            

            USE_SQUARES = 1
            if USE_SQUARES:
                if any(ind_lb):
                    d[ind_lb] -= lb[ind_lb]# d/dx((x-lb)^2) for violated constraints
                if any(ind_ub):
                    d[ind_ub] += ub[ind_ub]# d/dx((x-ub)^2) for violated constraints
                if any(ind_lin_ineq):
                    # d/dx((Ax-b)^2)
                    Ind_lin_ineq = where(ind_lin_ineq)[0]
                    b = p.b[Ind_lin_ineq]
                    if hasattr(p, '_A'):
                        a = p._A[Ind_lin_ineq] 
                        tmp = a._mul_sparse_matrix(csr_matrix((self.x, (arange(n), zeros(n))), shape=(n, 1))).toarray().flatten() - b 
                        
                        #tmp = a._mul_sparse_matrix(csr_matrix((self.x, reshape(p.n, 1))).toarray().flatten() - b 
                        d += a.T._mul_sparse_matrix(tmp.reshape(tmp.size, 1)).A.flatten()
                        #d += dot(a.T, dot(a, self.x)  - b) 
                    else:
                        a = p.A[ind_lin_ineq] 
                        d += dot(a.T, dot(a, self.x)  - b) # d/dx((Ax-b)^2)
                if any(ind_lin_eq):
                    #Ind_lin_eq = where(ind_lin_eq)[0]
                    if isspmatrix(p.Aeq):
                        p.err('this solver is not ajusted to handle sparse Aeq matrices yet')
                    #self.p.err('nonzero threshold is not ajusted with lin eq yet')
                    aeq = p.Aeq#[Ind_lin_eq]
                    beq = p.beq#[Ind_lin_eq]
                    d += dot(aeq.T, dot(aeq, self.x)  - beq) # d/dx((Aeq x - beq)^2)
                    

#                self._all_lin_gradient = 2.0 * d
                self._all_lin_gradient = 2.0 * d / p.contol

            else:
                assert 0
                if any(ind_lb):
                    d[ind_lb] -= 1# d/dx(lb-x) for violated constraints
                if any(ind_ub):
                    d[ind_ub] += 1# d/dx(x-ub) for violated constraints
                if any(ind_lin_ineq):
                    # d/dx(Ax-b)
                    b = p.b[ind_lin_ineq]
                    if hasattr(p, '_A'):
                        d += (p._A[ind_lin_ineq]).sum(0).A.flatten()
                    else:
                        d += (p.A[ind_lin_ineq]).sum(0).flatten()
                if any(ind_lin_eq):
                    # currently for ralg it should be handled in dilation matrix
                    p.err('not implemented yet, if you see it inform OpenOpt developers')
#                    beq = p.beq[ind_lin_eq]
#                    if hasattr(p, '_Aeq'):
#                        tmp = p._Aeq[ind_lin_eq]
#                        ind_change = where()
#                        tmp
#                        d += ().sum(0).A.flatten()
#                    else:
#                        #d += (p.Aeq[ind_lin_eq]).sum(0).flatten()

#                    aeq = p.Aeq[ind_lin_eq]
#                    beq = p.beq[ind_lin_eq]
#                    d += dot(aeq.T, dot(aeq, self.x)  - beq) # 0.5*d/dx((Aeq x - beq)^2)
                self._all_lin_gradient = d
        return copy(self._all_lin_gradient)

    def sum_of_all_active_constraints_gradient(self):
        if not hasattr(self, '_sum_of_all_active_constraints_gradient'):
            p = self.p
            contol = p.contol
            x = self.x
            direction = self.all_lin_gradient()
            if p.solver.__name__ == 'ralg':
                new = 1
            elif p.solver.__name__ == 'gsubg':
                new = 0
            else:
                p.err('unhandled case in Point._getDirection')
                
            if p.userProvided.c:
                th = 0.0
                #th = contol / 2.0
                C = p.c(x)
                Ind = C>th
                ind = where(Ind)[0]
                activeC = asarray(C[Ind])# asarray and Ind for PyPy compatibility
                if len(ind) > 0:
                    tmp = p.dc(x, ind)

                    if new:
                        if tmp.ndim == 1 or min(tmp.shape) == 1:
                            if hasattr(tmp, 'toarray'): 
                                tmp = tmp.toarray()#.flatten()
                            if activeC.size == prod(tmp.shape):
                                activeC = activeC.reshape(tmp.shape)
                            tmp *= (activeC-th*(1.0-1e-15))/norm(tmp)
                        else:
                            if hasattr(tmp, 'toarray'):
                                tmp = tmp.toarray()
                            tmp *= ((activeC - th*(1.0-1e-15))/sqrt((tmp**2).sum(1))).reshape(-1, 1)
                            
                    if tmp.ndim > 1:
                        tmp = tmp.sum(0)
                    direction += (tmp.A if type(tmp) != ndarray else tmp).flatten()
            

            if p.userProvided.h:
                #th = 0.0
                th = contol / 2.0
                H = p.h(x)
                Ind1 = H>th
                ind1 = where(Ind1)[0]
                H1 = asarray(H[Ind1])# asarray and Ind1 for PyPy compatibility
                if len(ind1) > 0:
                    tmp = p.dh(x, ind1)
                    
                    if new:
                        if tmp.ndim == 1 or min(tmp.shape) == 1:
                            if hasattr(tmp, 'toarray'): 
                                tmp = tmp.toarray()#.flatten()
                            if H1.size == prod(tmp.shape):
                                H1 = H1.reshape(tmp.shape)
                            tmp *= (H1-th*(1.0-1e-15))/norm(tmp)
                        else:
                            if hasattr(tmp, 'toarray'):
                                tmp = tmp.toarray()
                            tmp *= ((H1 - th*(1.0-1e-15))/sqrt((tmp**2).sum(1))).reshape(-1, 1)
                    
                    if tmp.ndim > 1: 
                        tmp = tmp.sum(0)
                    direction += (tmp.A if isspmatrix(tmp) or hasattr(tmp, 'toarray') else tmp).flatten()
                ind2 = where(H<-th)[0]
                H2 = H[ind2]
                if len(ind2) > 0:
                    tmp = p.dh(x, ind2)
                    if new:
                        if tmp.ndim == 1 or min(tmp.shape) == 1:
                            if hasattr(tmp, 'toarray'): 
                                tmp = tmp.toarray()#.flatten()
                            if H2.size == prod(tmp.shape):
                                H2 = H2.reshape(tmp.shape)                                    
                            tmp *= (-H2-th*(1.0-1e-15))/norm(tmp)
                        else:
                            if hasattr(tmp, 'toarray'):
                                tmp = tmp.toarray()
                            tmp *= ((-H2 - th*(1.0-1e-15))/sqrt((tmp**2).sum(1))).reshape(-1, 1)
                    
                    if tmp.ndim > 1: 
                        tmp = tmp.sum(0)
                    direction -= (tmp.A if type(tmp) != ndarray else tmp).flatten()
            self._sum_of_all_active_constraints_gradient = direction
        return Copy(self._sum_of_all_active_constraints_gradient)


    def _getDirection(self, approach, currBestFeasPoint = None):
        if self.isFeas(altLinInEq=True):
            self.direction, self.dType = self.df(),'f'
            if type(self.direction) != ndarray: self.direction = self.direction.A.flatten()
            return self.direction.copy()
        else:
            if approach == 'all active':
                self.dType  = 'all active'
                self.direction = self.sum_of_all_active_constraints_gradient()
            else:
                maxRes, fname, ind = self.mr_alt(1, bestFeasiblePoint = currBestFeasPoint)
                if fname == 'all_lin':
                    d = self.all_lin_gradient()
                    self.dType = 'all_lin'
                elif fname == 'lin_eq':
                    self.p.err("kernel error, inform openopt developers")
                    #d = self.dmr()
                    #self.dType = 'lin_eq'
                elif fname == 'c':
                    d = self.dmr()
                    #if p.debug: assert array_equal(self.dc(ind).flatten(), self.dmr())
                    self.dType = 'c'
                elif fname == 'h':
                    d = self.dmr()#sign(self.h(ind))*self.dh(ind)
                    #if p.debug: assert array_equal(self.dh(ind).flatten(), self.dmr())
                    self.dType = 'h'
                else:
                    self.p.err('error in getDirection (unknown residual type ' + fname + ' ), you should report the bug')
                self.direction = d.flatten()

            if type(self.direction) != ndarray: self.direction = self.direction.A.flatten()
            
#            print 'currBestFeasPoint is not None:', (currBestFeasPoint is not None), 'self.f() > currBestFeasPoint.f():', self.f() > currBestFeasPoint.f()

            contol, fTol = self.p.contol, self.p.fTol
#            print self.f(),  currBestFeasPoint.f(), self.f() - fTol ==  currBestFeasPoint.f()
            
            if currBestFeasPoint is not None and self.f() + 0.25*fTol > currBestFeasPoint.f():
                
                #self.direction += ((self.f()-currBestFeasPoint.f())  * p.contol / nDF / fTol) * DF
#                self.direction += self.df() * (contol/fTol)       
                pass
                    
            return self.direction.copy() # it may be modified in ralg when some constraints coords are NaN




from numpy import asfarray, argmax, sign, inf, log10, dot
from openopt.kernel.ooMisc import norm
from openopt.kernel.baseSolver import baseSolver
from openopt import NSP
from openopt.kernel.setDefaultIterFuncs import IS_MAX_FUN_EVALS_REACHED, FVAL_IS_ENOUGH, SMALL_DELTA_X, SMALL_DELTA_F

class nssolve(baseSolver):
    __name__ = 'nssolve'
    __license__ = "BSD"
    __authors__ = 'Dmitrey Kroshko'
    __alg__ = "based on Naum Z. Shor r-alg"
    iterfcnConnected = True
    __optionalDataThatCanBeHandled__ = ['A', 'Aeq', 'b', 'beq', 'lb', 'ub', 'c', 'h']
    __isIterPointAlwaysFeasible__ = lambda self, p: p.isUC
    
    nspSolver = 'autoselect'
    __info__ = """
    Solves system of non-smooth or noisy equations
    via (by default) minimizing max residual using NSP solver (default UkrOpt.ralg).

    Can handle user-supplied gradient/subradient (p.df field)
    If the one is not available -
    splitting equations to separate functions is recommended
    (to speedup calculations):
    f = [func1, func2, ...] or f = ([func1, func2, ...)

    ns- can be interpreted as
    NonSmooth
    or NoiSy
    or Naum Shor (Ukrainian academician, my teacher, r-algorithm inventor)
    """

    def __init__(self):pass
    def __solver__(self, p):
        if SMALL_DELTA_X in p.kernelIterFuncs.keys(): p.kernelIterFuncs.pop(SMALL_DELTA_X)
        if SMALL_DELTA_F in p.kernelIterFuncs.keys(): p.kernelIterFuncs.pop(SMALL_DELTA_F)

        if self.nspSolver == 'autoselect':
            nspSolver = 'amsg2p' if p.isUC else 'ralg'
        else:
            nspSolver = self.nspSolver
        
#        nspSolver = 'ralg'

        way = 3 if nspSolver == 'ralg' else 2
       
        if way == 1:
            use2 = False
            f = lambda x: sum(abs(p.f(x)))
            def df(x):
                return dot(p.df(x),  sign(p.f(x)))
        elif way == 2:
            use2 = True
            f = lambda x: sum(p.f(x)**2)
            def df(x):
                return 2.0*dot(p.f(x),  p.df(x))
        elif way == 3:
            use2 = False
            f = lambda x: max(abs(p.f(x)))
            def df(x):
                F = p.f(x)
                ind = argmax(abs(F))
                return p.df(x, ind) * sign(F[ind])

        FTOL = p.ftol**2 if use2 else p.ftol
        def iterfcn(*args,  **kwargs):
            p2.primalIterFcn(*args,  **kwargs)
            p.xk = p2.xk.copy()
            Fk = norm(p.f(p.xk), inf)
            p.rk = p.getMaxResidual(p.xk)

            #TODO: ADD p.rk

            if p.nEvals['f'] > p.maxFunEvals:
                p.istop = p2.istop = IS_MAX_FUN_EVALS_REACHED
            elif p2.istop!=0:
                if Fk < FTOL and p.rk < p.contol:
                    p.istop = 15
                    msg_contol = '' if p.isUC else 'and contol '
                    p.msg = 'solution with required ftol ' + msg_contol+ 'has been reached'
                else: p.istop = p2.istop

            p.iterfcn()
            
            return p.istop


        p2 = NSP(f, p.x0, df=df, xtol = p.xtol/1e16, gtol = p.gtol/1e16,\
        A=p.A,  b=p.b,  Aeq=p.Aeq,  beq=p.beq,  lb=p.lb,  ub=p.ub, \
        maxFunEvals = p.maxFunEvals, fEnough = FTOL, maxIter=p.maxIter, iprint = -1, \
        maxtime = p.maxTime, maxCPUTime = p.maxCPUTime,  noise = p.noise, fOpt = 0.0)

        if p.userProvided.c:
            p2.c,  p2.dc = p.c,  p.dc
        if p.userProvided.h:
            p2.h,  p2.dh = p.h,  p.dh

        p2.primalIterFcn,  p2.iterfcn = p2.iterfcn, iterfcn

        if p.debug: p2.iprint = 1
        
        if nspSolver == 'ralg':
            if p.isUC: p2.ftol = p.ftol / 1e16
            else: p2.ftol = 0.0
        else:
            p2.ftol = 0.0
            p2.xtol = 0.0
            p2.gtol = 0.0
        if use2:
            p2.fTol = 0.5*p.ftol ** 2
        else:
            p2.fTol = 0.5*p.ftol
        r2 = p2.solve(nspSolver)
        #xf = fsolve(p.f, p.x0, fprime=p.df, xtol = p.xtol, maxfev = p.maxFunEvals)
        xf = r2.xf
        p.xk = p.xf = xf
        p.fk = p.ff = asfarray(norm(p.f(xf), inf)).flatten()
        #p.istop is defined in iterfcn
